/*--------------------------------------------------------------*/
/* Mikie (c)1984 Konami, arcade emulator.			*/
/* Written by Bruno VEDDER. 6809 Cpu emulation based on 	*/
/* Daniel Coulomb work (http://dcmo5.free.fr/), and modified for*/
/* Multi-Cpu and accurate cycles counting.			*/
/*--------------------------------------------------------------*/
/* Special thanks to Sean Riddle for helping me with color	*/
/* decoding, and mame team for their unvaluable sources.	*/
/*--------------------------------------------------------------*/
/* This source is provided as is, without warranty. Feel free   */
/* to use this code, and modify it for non profit/commercial	*/
/* purpose. If you have any questions about this program don't	*/
/* hesitate and drop me a mail at bruno.vedder@emu-france.com	*/
/* Note: Don't ask for ROM, they won't be provided !!!		*/
/*______________________________________________________________*/
/*								*/
/* ---[Bruno's Coding Zone]--------------------------		*/
/*    URL http://bcz.emu-france.com/  				*/
/*    Mail: bruno.vedder@emu-france.com				*/
/*______________________________________________________________*/
/*								*/

#ifndef M6809_H
#define M6809_H
#define MNEMONIC_LEN	10			/* Mnemonic string lenght 	*/
#define DESCRIPTION_LEN	16			/* Mnemonic description len	*/
#define MNEMO_HELP_LEN	32			/* Help on mnemonic effects 	*/
#define DASM_STR_LEN	128			/* Lenght of a return string of dissassembly 	*/
#define DUMP_STR_LEN	32			/* Lenght of a return string of Dump 		*/
#define OPERAND_STR_LEN 32			/* Lenght of a return string of operand list 	*/

/*---------- Link Adressing mode to its display function ----------*/

#define INHERENT	AM_Inherent		/* Adressing mode INHERENT 	*/
#define IMMEDIAT_8	AM_Immediat_8		/* Immediat 8 Bit		*/
#define IMMEDIAT_16	AM_Immediat_16		/* Immediate 16 Bit		*/
#define BRANCH_REL_8	AM_Branch_Rel_8		/* Branch relative 8 bit. +127 -128 bytes move */
#define BRANCH_REL_16	AM_Branch_Rel_16	/* Branch relative 16 bit. +32767 -32768 bytes move */
#define DIRECT		AM_Direct		/* Via DP register for msb			*/
#define EXTENDED	AM_Extended		/* in C =  *(u8*) 				*/
#define INDEXED		AM_Indexed		/* Indexed mode */
#define ILLEGAL		AM_Illegal		/* Illegal addressing mode */

/*---------- Defines used for Inherent Adressing Mode ----------*/

#define OP_EXG		0x1e			/* EXG Opcode 	*/
#define OP_TFR		0x1f			/* TFR Opcode 	*/
#define OP_PSHS		0x34			/* PSHS Opcode  */
#define OP_PULS		0x35			/* PULS Opcode  */
#define OP_PSHU		0x36			/* PSHU Opcode  */
#define OP_PULU		0x37			/* PULU Opcode  */
#define OP_CWAI		0x3c			/* CWAI Opcode  */
#define OP_SWI		0x3f			/* SWIx Opcode  */



/*---------------	Typedef and structures	     ---------------*/

typedef unsigned char 	u8;			/* unsigned 8 bit */
typedef signed char 	s8;			/* signed 8 bit */
typedef unsigned short 	u16;			/* unsigned 16 bit */
typedef signed short 	s16;			/* signed 16 bit */
typedef unsigned int 	u32;			/* unsigned 32 bit */
typedef signed int 	s32;			/* signed 32 bit */

typedef struct
{
u16 D;						/* A and B register = D 16 bit register */
u16 DPR;					/* Direct Page Register 		*/
u8  CC;						/* Condition Code register 		*/
u16 X;						/* X register 				*/
u16 Y;						/* Y register 				*/
u16 U;						/* User Stack register 			*/
u16 S;						/* System Stack register 		*/
u16 PC;						/* Program Counter register		*/
u16 EMP;					/* Extended mode ptr (not cpu part)*/
s8 Dum;						/* Dum variable for invalide TFR/EXG address.*/
u16 BreakPoint;					/* Not a cpu part :-) */
u32 Cycles;					/* Cycle counter */
u32 IrqF;					/* Irq frequency */
u16 *pReg16[4];					/* Ptr on X,Y,U,S. Initialised in CPUReset*/
void *RgPtr[16];				/* Reg ptr used by tfr/exg instruction*/
u8  (*ReadMem8) (u16 Adr);			/* Funcs use to make byte read operation.    */
u16 (*ReadMem16)(u16 Adr);			/* Add decoding, I/O map are performed here. */
void (*WriteMem8) (u16 Adr,u8   Val);		/* Same but for 16 bits operations.  	*/
void (*WriteMem16)(u16 Adr,u16 Val);		/* This is for multi-processors use. 	*/
} M6809CPU;

typedef struct
{
u8 Opcode;					/* Operande Opcode 			*/
u8 Clock;					/* Number of cycle duration 		*/
u8 Size;					/* Opcode+Operande size		 	*/
int  (*AddrMode)();				/* Addressing mode dep. operand decoder	*/
char Mnemonic [MNEMONIC_LEN];			/* Mnemonic string 			*/
char Description [DESCRIPTION_LEN];		/* Mnemonic short description 		*/
char MnemoHelp [MNEMO_HELP_LEN];		/* More complete description  		*/
} M6809Opcode;

extern M6809Opcode OpcodeTable [256];
extern M6809Opcode OpcodeTable10 [256];
extern M6809Opcode OpcodeTable11 [256];
#endif