/*--------------------------------------------------------------*/
/* Mikie (c)1984 Konami, arcade emulator.			*/
/* Written by Bruno VEDDER. 6809 Cpu emulation based on 	*/
/* Daniel Coulomb work (http://dcmo5.free.fr/), and modified for*/
/* Multi-Cpu and accurate cycles counting.			*/
/*--------------------------------------------------------------*/
/* Special thanks to Sean Riddle for helping me with color	*/
/* decoding, and mame team for their unvaluable sources.	*/
/*--------------------------------------------------------------*/
/* This source is provided as is, without warranty. Feel free   */
/* to use this code, and modify it for non profit/commercial	*/
/* purpose. If you have any questions about this program don't	*/
/* hesitate and drop me a mail at bruno.vedder@emu-france.com	*/
/* Note: Don't ask for ROM, they won't be provided !!!		*/
/*______________________________________________________________*/
/*								*/
/* ---[Bruno's Coding Zone]--------------------------		*/
/*    URL http://bcz.emu-france.com/  				*/
/*    Mail: bruno.vedder@emu-france.com				*/
/*______________________________________________________________*/
/*								*/

/* word storing: Little-indian LSB=0 MSB=Add+1 Big-indian LSB=Add+1 MSB=0 */
#define LSB  0
#define MSB  1

/*----- CPU CC Register FLAG DEFINES -----*/
#define  CF  0x01  /* carry */
#define  VF  0x02  /* overflow */
#define  ZF  0x04  /* zero */
#define  NF  0x08  /* negative */
#define  INF 0x10  /* int mask */
#define  HF  0x20  /* half-carry */
#define  FF  0x40  /* firq mask */
#define  EF  0x80  /* extended registers save */
#define  EF0 0x7f  /* EF =0 mask */	
#define  NV 0x0a   /* negative & overflow mask */
#define  ZC 0x05   /* zero & carry mask*/
#define NZC 0x0e   /* negative & zero & carry mask */

/*----- 	conditional jumps :	-----

   C BCC BCS      Z BNE BEQ      V BVC BVS     NZV BGT BLE
   0 yes no       0 yes no       0 yes no      000 yes no
   1 no  yes      1 no  yes      1 no  yes     100 no  yes
                                               010 no  yes
   N BPL BMI     ZC BHI BLS     NV BGE BLT     001 no  yes
   0 yes no      00 yes no      00 yes no      110 no  no
   1 no  yes     10 no  yes     10 no  yes     101 yes no
                 01 no  yes     01 no  yes     011 no  no
                 11 no  no      11 yes no      111 no  yes
*/
#define BHI (RegCC&ZC)==0			/* Branch if Higher cond.*/
#define BLS (RegCC&ZC)==ZF||(RegCC&ZC)==CF
#define BCC (RegCC&CF)==0
#define BCS (RegCC&CF)==CF
#define BNE (RegCC&ZF)==0			/* Branch if Not Equal.*/
#define BEQ (RegCC&ZF)==ZF			/* Branch if Equal.*/
#define BVC (RegCC&VF)==0
#define BVS (RegCC&VF)==VF
#define BPL (RegCC&NF)==0
#define BMI (RegCC&NF)==NF
#define BGE (RegCC&NV)==0||(RegCC&NV)==NV
#define BLT (RegCC&NV)==NF||(RegCC&NV)==VF
#define BGT (RegCC&NZC)==0||(RegCC&0x0e)==NV
#define BLE (RegCC&NZC)==NF||(RegCC&NZC)==ZF||(RegCC&NZC)==VF||(RegCC&NZC)==NZC

/*----- Register naming simplification -----*/

#define RegX	CPU->X
#define Xh	*((char *)&CPU->X+MSB)
#define Xl	*((char *)&CPU->X+LSB)
#define RegY	CPU->Y
#define Yh	*((char *)&CPU->Y+MSB)
#define Yl	*((char *)&CPU->Y+LSB)
#define RegU	CPU->U
#define Uh	*((char *)&CPU->U+MSB)
#define Ul	*((char *)&CPU->U+LSB)
#define RegS	CPU->S
#define Sh	*((char *)&CPU->S+MSB)
#define Sl	*((char *)&CPU->S+LSB)
#define RegPC	CPU->PC
#define PCh	*((char *)&CPU->PC+MSB)
#define PCl	*((char *)&CPU->PC+LSB)
#define RegCC	CPU->CC
#define RegD	(s16)CPU->D
#define RegA	*((char *) &CPU->D + MSB)  	/* A is MSB of D register */
#define RegB	*((char *) &CPU->D + LSB)	/* B is LSB of D register */
#define RegDum  CPU->Dum			/* Dum register for invalid tfr/Exg addressing*/
#define RgPtr   CPU->RgPtr

#define RegDP	*((char *)&CPU->DPR+MSB)	/* DP is MSB of a 16bit ptr */
#define RegDD	*((char *)&CPU->DPR+LSB)	/* DD is the lower part of DPR, usualy imm readed */
#define VDPR  	CPU->DPR			/* Full Direct page use to compute access address*/

#define VEMP	CPU->EMP			/* Virtual Ext mode ptr */
#define VEMPh	*((char *)&CPU->EMP+MSB)
#define VEMPl	*((char *)&CPU->EMP+LSB)

#define MGETD	Mgetd(CPU)			/* Call to Mgetd simplification */
#define MGETE	Mgete(CPU)			/* Call to Mgete simplification */
#define MGETI	Mgeti(CPU,C) 			/* Call to Mgeti simplification */

#define Rd8(a)	(s8)CPU->ReadMem8(a)		/* ReadMem8 CPU simplification */
#define Rd16(a)	(s16)CPU->ReadMem16(a)		/* ReadMem16 CPU simplification */
#define Wr8(a,c)  CPU->WriteMem8 (a,c)		/* WriteMem8 CPU simplification*/
#define Wr16(a,c) CPU->WriteMem16 (a,c)		/* WriteMem8 CPU simplification*/
