/*--------------------------------------------------------------*/
/* Mikie (c)1984 Konami, arcade emulator.			*/
/* Written by Bruno VEDDER. 6809 Cpu emulation based on 	*/
/* Daniel Coulomb work (http://dcmo5.free.fr/), and modified for*/
/* Multi-Cpu and accurate cycles counting.			*/
/*--------------------------------------------------------------*/
/* Special thanks to Sean Riddle for helping me with color	*/
/* decoding, and mame team for their unvaluable sources.	*/
/*--------------------------------------------------------------*/
/* This source is provided as is, without warranty. Feel free   */
/* to use this code, and modify it for non profit/commercial	*/
/* purpose. If you have any questions about this program don't	*/
/* hesitate and drop me a mail at bruno.vedder@emu-france.com	*/
/* Note: Don't ask for ROM, they won't be provided !!!		*/
/*______________________________________________________________*/
/*								*/
/* ---[Bruno's Coding Zone]--------------------------		*/
/*    URL http://bcz.emu-france.com/  				*/
/*    Mail: bruno.vedder@emu-france.com				*/
/*______________________________________________________________*/
/*								*/
#include <stdio.h>
#include <string.h>
#include "6809emu.h"
#include "6809emu-def.h"

#define IRQ_FREQUENCY  20833		/* cycles nbr for 60Hz with 1.25Mhz  CPU */

/*----- Tst and compare instructions -----*/

void Tstc(M6809CPU *CPU,s8 c);
void Tstw(M6809CPU *CPU,s16 w);
void Cmpc(M6809CPU *CPU,s8 reg,s8 c);
void Cmpw(M6809CPU *CPU,s16 reg,s16 w);

/*----- Neg, Inc, Dec, Clr, Com  instructions -----*/

s8 Neg(M6809CPU *CPU, s8 c);
s8 Com(M6809CPU *CPU,s8 c);
s8 Inc(M6809CPU *CPU,s8 c);
s8 Dec(M6809CPU *CPU,s8 c);
s8 Clr(M6809CPU *CPU);

/*----- Lsr, Ror, Rol, Asr, Asl Instructions -----*/

s8 Lsr(M6809CPU *CPU,u8 c);
s8 Ror(M6809CPU *CPU,u8 c);
s8 Rol(M6809CPU *CPU,u8 c);
s8 Asr(M6809CPU *CPU,u8 c);
s8 Asl(M6809CPU *CPU,u8 c);

/*----- Mul, Adc, Addc,Adw, Subc, Sbc, Subw, Daa -----*/

void Mul(M6809CPU *CPU);
void Addc(M6809CPU *CPU,s8 *reg, s8 c);
void Adc(M6809CPU *CPU,s8 *reg, s8 c);
void Addw(M6809CPU *CPU,s16 *reg, s16 w);
void Subc(M6809CPU *CPU,s8 *reg, s8 c);
void Sbc(M6809CPU *CPU,s8 *reg, s8 c);
void Subw(M6809CPU *CPU,s16 *reg, s16 w);
s8 Daa(M6809CPU *CPU,u8 c);

/*----- Puls,Pulu,Pshs,Pshu -----*/

void Pshu (M6809CPU *CPU,u8 c);
void Pshs (M6809CPU *CPU,u8 c);
void Pulu (M6809CPU *CPU,u8 c);
void Puls (M6809CPU *CPU,u8 c);

/*----- Memory Access -----*/

s16 Mgetd(M6809CPU *CPU);	/* Direct page 		*/ 
s16 Mgete(M6809CPU *CPU);	/* Extended, imm16 	*/
s16 Mgeti(M6809CPU *CPU,u8 *C);	/* Indexed mode *C is a counter ptr to add index extra cycles */

/*----- EXG / TFR -----*/

void Exg(M6809CPU *CPU,u8 c);
void Tfr(M6809CPU *CPU,u8 c);

/*----- IRQ , FIRQ -----*/

void Irq (M6809CPU *CPU,u16 ad);
void Firq (M6809CPU *CPU,u16 ad);

/*----------------------------------------------------------------------*/
/* CPU Register Initialisation. All registers are setup.		*/
/*----------------------------------------------------------------------*/
void Reset6809 (M6809CPU *CPU)
{
RegPC = CPU->ReadMem16 (0xFFFE);
RegX =  0x0;
RegY =  0x0;
RegU =  0x0000;
RegS =  0x8000;
RegCC=  0x00;	
RegDP=  0x00;
RegA =  0x0;  	/* Equiv to RegD =00 */
RegB=   0x0;	/* Equiv to RegD =00 */
CPU->Cycles =0;
CPU->BreakPoint = 0x0000;
CPU->IrqF = IRQ_FREQUENCY;

CPU->pReg16 [0] = & RegX;
CPU->pReg16 [1] = & RegY;
CPU->pReg16 [2] = & RegU;
CPU->pReg16 [3] = & RegS;

RgPtr [0] = &RegD;
RgPtr [1] = &RegX;
RgPtr [2] = &RegY;
RgPtr [3] = &RegU;
RgPtr [4] = &RegS;
RgPtr [5] = &RegPC;
RgPtr [6] = &RegDum;
RgPtr [7] = &RegDum;
RgPtr [8] = &RegA;
RgPtr [9] = &RegB;
RgPtr [10] = &RegCC;
RgPtr [11] = &RegDP;
RgPtr [12] = &RegDum;
RgPtr [13] = &RegDum;
RgPtr [14] = &RegDum;
RgPtr [15] = &RegDum;
}

/*----------------------------------------------------------------------*/
/* Execute one M6809 Instruction.					*/
/* Input: CPU is the concerned CPU :-)					*/
/*	  nb_cycle = cycle instruction duration return value.		*/
/* Return: 00 Ok, 0x11 0x12 illegal opcode				*/	
/*----------------------------------------------------------------------*/
u8 Exec6809 (M6809CPU *CPU,u8 *C)
{
u8 r=0;	/* OK by default */
u16 Dummy; /* Used by BSR 0x8d Opcode */

*C=0;	/* Instruction Cycles nbr */
switch (CPU->ReadMem8(RegPC++)) /* Read as u8 ! */
	{
	case 0x00: *C=6; MGETD;Wr8(VDPR,Neg(CPU,Rd8(VDPR)));break;		/* NEG /$ */	
	case 0x01: *C=1; RegPC++;/* Undocumented BRN Opcode */ break;		/* Undoc */	
	case 0x03: *C=6; MGETD;Wr8(VDPR,Com(CPU,Rd8(VDPR)));break;		/* COM /$ */	
	case 0x04: *C=6; MGETD;Wr8(VDPR,Lsr(CPU,Rd8(VDPR)));break;		/* LSR /$ */		
	case 0x06: *C=6; MGETD;Wr8(VDPR,Ror(CPU,Rd8(VDPR)));break;		/* ROR /$ */		
	case 0x07: *C=6; MGETD;Wr8(VDPR,Asr(CPU,Rd8(VDPR)));break;		/* ASR /$ */		
	case 0x08: *C=6; MGETD;Wr8(VDPR,Asl(CPU,Rd8(VDPR)));break;		/* ASL /$ */	
	case 0x09: *C=6; MGETD;Wr8(VDPR,Rol(CPU,Rd8(VDPR)));break;		/* ROL /$ */	
	case 0x0A: *C=6; MGETD;Wr8(VDPR,Dec(CPU,Rd8(VDPR)));break;		/* DEC /$ */		
	case 0x0C: *C=6; MGETD;Wr8(VDPR,Inc(CPU,Rd8(VDPR)));break;		/* INC /$ */	
	case 0x0D: *C=6; Tstc (CPU,Rd8(MGETD));break;				/* TST /$ */
	case 0x0E: *C=6; RegPC=MGETD;break;					/* JMP /$ */
	case 0x0F: *C=6; Wr8(MGETD,Clr(CPU));break;				/* CLR /$ */
	case 0x10:
	switch (CPU->ReadMem8 (RegPC++)) /* Read as u8! */
		{ /* Opcode table by 0x10 prefixed */
		case 0x21: *C=5;RegPC+=2;break;					/* LBRN xx /Nop */
		case 0x22: *C=5;if (BHI)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBHI */
		case 0x23: *C=5;if (BLS)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBLS */
		case 0x24: *C=5;if (BCC)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBCC */
		case 0x25: *C=5;if (BCS)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBCS */
		case 0x26: *C=5;if (BNE)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBNE */
		case 0x27: *C=5;if (BEQ)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBEQ */
		case 0x28: *C=5;if (BVC)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBVC */
		case 0x29: *C=5;if (BVS)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBVS */
		case 0x2a: *C=5;if (BPL)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBPL */
		case 0x2b: *C=5;if (BMI)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBMI */
		case 0x2c: *C=5;if (BGE)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBGE */
		case 0x2d: *C=5;if (BLT)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBLT */
		case 0x2e: *C=5;if (BGT)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBGT */
		case 0x2f: *C=5;if (BLE)RegPC+=Rd16(RegPC);RegPC+=2;break;	/* LBLE */
		case 0x83: *C=5;Cmpw (CPU,RegD,MGETE);break;			/* CMPD #$*/
		case 0x8c: *C=5;Cmpw(CPU,RegY, MGETE); break;    		/* CMPY #$ */
		case 0x8e: *C=4;Tstw(CPU,RegY= MGETE); break; 			/* LDY  #$ */
		case 0x93: *C=7;Cmpw(CPU,RegD,Rd16(MGETD));break;		/* CMPD /$ */
		case 0x9c: *C=7;Cmpw(CPU,RegY,Rd16(MGETD));break;		/* CMPY /$ */
		case 0x9e: *C=6;Tstw(CPU,RegY=Rd16(MGETD));break;		/* LDY  /$ */
		case 0x9f: *C=6;Wr16(MGETD,RegY);Tstw(CPU,RegY);break;		/* STY  /$ */
		case 0xa3: *C=7;Cmpw(CPU,RegD,Rd16(MGETI));break;		/* CMPD IX */
		case 0xac: *C=7;Cmpw(CPU,RegY,Rd16(MGETI));break;		/* CMPY IX */
		case 0xae: *C=6;Tstw(CPU,RegY=Rd16(MGETI));break;		/* LDY  IX */
		case 0xaf: *C=6;Wr16(MGETI,RegY);Tstw(CPU,RegY);break;		/* STY  IX */
		case 0xb3: *C=8;Cmpw (CPU,RegD,Rd16(MGETE));break;		/* CMPD $ */
		case 0xbc: *C=8;Cmpw (CPU,RegY,Rd16(MGETE));break;		/* CMPY $ */
		case 0xbe: *C=7;Tstw (CPU,RegY=Rd16(MGETE));break;		/* LDY  $ */
		case 0xbf: *C=7;Wr16 (MGETE,RegY);Tstw(CPU,RegY);break;		/* STY $ */
		case 0xce: *C=4;Tstw(CPU,RegS=MGETE);break;      		/* LDS  #$ */
		case 0xde: *C=6;Tstw(CPU,RegS=Rd16(MGETD));break;		/* LDS  /$ */
		case 0xdf: *C=6;Wr16 (MGETD,RegS);Tstw(CPU,RegS);break;		/* STS  /$ */
		case 0xee: *C=6;Tstw(CPU,RegS=Rd16(MGETI));break;		/* LDS  IX */
		case 0xef: *C=6;Wr16(MGETI,RegS);Tstw(CPU,RegY);break;		/* STS  IX */
		case 0xfe: *C=7;Tstw(CPU,RegS=Rd16(MGETE));break;		/* LDS $   */
		case 0xff: *C=7;Wr16 (MGETE,RegS);Tstw (CPU,RegS);break;	/* STS $   */
		default:r=0x10;break;
		}
	break;
	case 0x11:
	switch (CPU->ReadMem8 (RegPC++)) /* Read as u8! */
		{
		case 0x83: *C=5;Cmpw(CPU,RegU,MGETE);break;			/* CMPU #$ */
		case 0x8c: *C=5;Cmpw(CPU,RegS,MGETE);break;			/* CMPS #$ */
		case 0x93: *C=7;Cmpw(CPU,RegU,Rd16(MGETD));break;		/* CMPU /$ */
		case 0x9c: *C=7;Cmpw(CPU,RegS,Rd16(MGETD));break;		/* CMPS /$ */
		case 0xa3: *C=7;Cmpw(CPU,RegU,Rd16(MGETI));break;		/* CMPU IX */
		case 0xac: *C=7;Cmpw(CPU,RegS,Rd16(MGETI));break;		/* CMPS IX */
		case 0xb3: *C=8;Cmpw(CPU,RegU,Rd16(MGETE));break;		/* CMPU $  */
		case 0xbc: *C=8;Cmpw(CPU,RegS,Rd16(MGETE));break;		/* CMPS $  */
		default:r=0x11;break;
		}
	break;
	case 0x12: *C=2; break;							/* NOP */
	case 0x13: *C=2; break;							/* SYNC */	
	case 0x16: *C=5;RegPC+=Rd16 (RegPC)+2; break;				/* LBRA #$ */
	case 0x17: *C=9;MGETE; Pshs (CPU,0x80); RegPC+=VEMP;break;		/* LBSR */
	case 0x19: *C=2;RegA=Daa(CPU,RegA);break;				/* DAA */
	case 0x1a: *C=3;RegCC |=Rd8(RegPC++);break; 				/* ORCC #$ */
	case 0x1c: *C=3;RegCC &=Rd8(RegPC++);break; 				/* ANDCC #$ */
	case 0x1d: *C=2;Tstw (CPU,RegD = RegB);break;	/* VERIFY !! */		/* SEX Sign Extend*/
	case 0x1e: *C=8;Exg(CPU,Rd8(RegPC++));break;				/* EXG */
	case 0x1f: *C=7;Tfr(CPU,Rd8(RegPC++));break;				/* TFR */
	case 0x20: *C=3;RegPC +=Rd8(RegPC)+1;break;				/* BRA xx */
	case 0x21: *C=3;RegPC++;break;						/* BRN xx /Nop */
	case 0x22: *C=3;if (BHI)RegPC+=Rd8(RegPC);RegPC++;break;		/* BHI */
	case 0x23: *C=3;if (BLS)RegPC+=Rd8(RegPC);RegPC++;break;		/* BLS */
	case 0x24: *C=3;if (BCC)RegPC+=Rd8(RegPC);RegPC++;break;		/* BCC */
	case 0x25: *C=3;if (BCS)RegPC+=Rd8(RegPC);RegPC++;break;		/* BCS */
	case 0x26: *C=3;if (BNE)RegPC+=Rd8(RegPC);RegPC++;break;		/* BNE */
	case 0x27: *C=3;if (BEQ)RegPC+=Rd8(RegPC);RegPC++;break;		/* BEQ */
	case 0x28: *C=3;if (BVC)RegPC+=Rd8(RegPC);RegPC++;break;		/* BVC */
	case 0x29: *C=3;if (BVS)RegPC+=Rd8(RegPC);RegPC++;break;		/* BVS */
	case 0x2a: *C=3;if (BPL)RegPC+=Rd8(RegPC);RegPC++;break;		/* BPL */
	case 0x2b: *C=3;if (BMI)RegPC+=Rd8(RegPC);RegPC++;break;		/* BMI */
	case 0x2c: *C=3;if (BGE)RegPC+=Rd8(RegPC);RegPC++;break;		/* BGE */
	case 0x2d: *C=3;if (BLT)RegPC+=Rd8(RegPC);RegPC++;break;		/* BLT */
	case 0x2e: *C=3;if (BGT)RegPC+=Rd8(RegPC);RegPC++;break;		/* BGT */
	case 0x2f: *C=3;if (BLE)RegPC+=Rd8(RegPC);RegPC++;break;		/* BLE */

	case 0x30: *C=4;RegX=MGETI;if (RegX) RegCC&=0xfb; else RegCC |= ZF; break;  /* LEAX    */
  	case 0x31: *C=4;RegY=MGETI;if (RegY) RegCC&=0xfb; else RegCC |= ZF; break;  /* LEAY    */
	case 0x32: *C=4; RegS = MGETI;break; 					/* LEAS */
	case 0x33: *C=4; RegU = MGETI;break; 					/* LEAU */
	case 0x34: *C=5;Pshs (CPU,Rd8(RegPC++)); break;				/* PSHS */
	case 0x35: *C=5;Puls (CPU,Rd8(RegPC++)); break;				/* PULS */
	case 0x36: *C=5;Pshu (CPU,Rd8(RegPC++)); break;				/* PSHU */
	case 0x37: *C=5;Pulu (CPU,Rd8(RegPC++)); break;				/* PULU */
	case 0x39: *C=5;Puls (CPU,0x80);break;					/* RTS */
	case 0x3a: *C=3;RegX+=(u8)RegB;break;					/* ABX */
	case 0x3b: *C=6;Puls(CPU,1);if(RegCC&EF)Puls(CPU,0xFE);else Puls(CPU,0x80);break; /* RTI */
	case 0x3c: *C=21;RegCC&=Rd8(RegPC++);RegCC|=EF;break;			/* CWAI */
	case 0x3d: *C=11;Mul (CPU);break;					/* MUL */
	/* RESET Instruction Here */
	case 0x3f: *C=19;Irq (CPU,0xFFFA);break;					/* SWI 1*/
	case 0x40: *C=2;RegA = Neg(CPU,RegA); break;  	                        /* NEGA */	
	/* 41 UNDOC  */
	case 0x43: *C=7;RegA = Com(CPU,RegA); break;  	                        /* COMA */	
	case 0x44: *C=2;RegA = Lsr(CPU,RegA); break;  	                        /* LSRA */	
	case 0x46: *C=2;RegA = Ror(CPU,RegA); break;  	                        /* RORA */		
	case 0x47: *C=2;RegA = Asr(CPU,RegA); break;  	                        /* ASRA */			
	case 0x48: *C=2;RegA = Asl(CPU,RegA); break;  	                        /* ASLA */			
	case 0x49: *C=2;RegA = Rol(CPU,RegA); break;  	                        /* ROLA */		
	case 0x4a: *C=2;RegA = Dec(CPU,RegA); break;  	                        /* DECA */		
	case 0x4c: *C=2;RegA = Inc(CPU,RegA); break;  	                        /* INCA */	
	case 0x4d: *C=2;Tstc(CPU,RegA);break;					/* TSTA */
	case 0x4f: *C=2;RegA = Clr(CPU); break;                                 /* CLRA */
	
	case 0x50: *C=2;RegB = Neg(CPU,RegB); break;  	                        /* NEGB */
	case 0x53: *C=7;RegB = Com(CPU,RegB); break;  	                        /* COMB */	
	case 0x54: *C=2;RegB = Lsr(CPU,RegB); break;  	                        /* LSRB */	
	case 0x56: *C=2;RegB = Ror(CPU,RegB); break;  	                        /* RORB */		
	case 0x57: *C=2;RegB = Asr(CPU,RegB); break;  	                        /* ASRB */			
	case 0x58: *C=2;RegB = Asl(CPU,RegB); break;  	                        /* ASLB */			
	case 0x59: *C=2;RegB = Rol(CPU,RegB); break;  	                        /* ROLB */		
	case 0x5a: *C=2;RegB = Dec(CPU,RegB); break;  	                        /* DECB */		
	case 0x5c: *C=2;RegB = Inc(CPU,RegB); break;  	                        /* INCB */	
	case 0x5d: *C=2;Tstc(CPU,RegB);break;					/* TSTB */
	case 0x5f: *C=2;RegB = Clr(CPU); break;                                 /* CLRB */

  	case 0x60: *C=6;Dummy=MGETI; Wr8(Dummy, Neg(CPU,Rd8(Dummy))); break;  		/* NEG  IX */
  	case 0x63: *C=6;Dummy=MGETI; Wr8(Dummy, Com(CPU,Rd8(Dummy))); break;    	/* COM  IX */
  	case 0x64: *C=6;Dummy=MGETI; Wr8(Dummy, Lsr(CPU,Rd8(Dummy))); break;    	/* LSR  IX */
  	case 0x66: *C=6;Dummy=MGETI; Wr8(Dummy, Ror(CPU,Rd8(Dummy))); break;    	/* ROR  IX */
  	case 0x67: *C=6;Dummy=MGETI; Wr8(Dummy, Asr(CPU,Rd8(Dummy))); break;    	/* ASR  IX */
  	case 0x68: *C=6;Dummy=MGETI; Wr8(Dummy, Asl(CPU,Rd8(Dummy))); break;    	/* ASL  IX */
  	case 0x69: *C=6;Dummy=MGETI; Wr8(Dummy, Rol(CPU,Rd8(Dummy))); break;    	/* ROL  IX */
  	case 0x6a: *C=6;Dummy=MGETI; Wr8(Dummy, Dec(CPU,Rd8(Dummy))); break;    	/* DEC  IX */
  	case 0x6c: *C=6;Dummy=MGETI; Wr8(Dummy, Inc(CPU,Rd8(Dummy))); break;    	/* INC  IX */
  	case 0x6d: *C=6;Tstc(CPU,Rd8(MGETI));break;					/* TST  IX */
	case 0x6e: *C=3;RegPC=MGETI;break;						/* JMP  IX */
	case 0x6f: *C=6;Wr8(MGETI,Clr(CPU));break;					/* CLR 	IX */
	
	case 0x70: *C=7;MGETE;Wr8(VEMP,Neg(CPU,Rd8(VEMP)));break;		/* NEG $ */
	case 0x73: *C=7;MGETE;Wr8(VEMP,Com(CPU,Rd8(VEMP)));break;		/* COM $ */
	case 0x74: *C=7;MGETE;Wr8(VEMP,Lsr(CPU,Rd8(VEMP)));break;		/* LSR $ */
	case 0x76: *C=7;MGETE;Wr8(VEMP,Ror(CPU,Rd8(VEMP)));break;		/* ROR $ */
	case 0x77: *C=7;MGETE;Wr8(VEMP,Asr(CPU,Rd8(VEMP)));break;		/* ASR $ */
	case 0x78: *C=7;MGETE;Wr8(VEMP,Asl(CPU,Rd8(VEMP)));break;		/* ASL $ */
	case 0x79: *C=7;MGETE;Wr8(VEMP,Rol(CPU,Rd8(VEMP)));break;		/* ROL $ */
	case 0x7a: *C=7;MGETE;Wr8(VEMP,Dec(CPU,Rd8(VEMP)));break;		/* DEC $ */
	case 0x7c: *C=7;MGETE;Wr8(VEMP,Inc(CPU,Rd8(VEMP)));break;		/* INC $ */
	case 0x7d: *C=7;Tstc(CPU,Rd8(MGETE));break;				/* TST $ */
	case 0x7e: *C=3;RegPC = Rd16(RegPC);break;				/* JMP $ */
	case 0x7f: *C=7;Wr8(MGETE,Clr(CPU));break;				/* CLR $ */
	
	case 0x80: *C=2;Subc (CPU,&RegA,Rd8 (RegPC++));break;			/* SUBA #$ */
	case 0x81: *C=2;Cmpc(CPU,RegA, Rd8(RegPC++)); break; 	                /* CMPA #$ */
	case 0x82: *C=2;Sbc (CPU,&RegA,Rd8 (RegPC++));break;			/* SBCA #$ */	
	case 0x83: *C=4;Subw(CPU,&RegD,MGETE);break;				/* SUBD #$ */
	case 0x84: *C=2;Tstc(CPU,RegA &= Rd8(RegPC++));break;			/* ANDA #$ */
	case 0x85: *C=2;Tstc(CPU,RegA &  Rd8(RegPC++));break;			/* BITA #$ */
	case 0x86: *C=2;Tstc(CPU,RegA=Rd8(RegPC++));break;	  	        /* LDA  #$ */
	case 0x88: *C=2;Tstc(CPU,RegA ^= Rd8(RegPC++));break;			/* EORA #$ */
	case 0x89: *C=2;Adc (CPU,&RegA,Rd8(RegPC++));break;			/* ADCa #$ */	
	case 0x8a: *C=2;Tstc(CPU,RegA |= Rd8(RegPC++));break;			/* ORA  #$ */
	case 0x8b: *C=2;Addc(CPU,&RegA,Rd8(RegPC++));break;			/* ADDA #$ */	
	case 0x8c: *C=4;MGETE;Cmpw(CPU,RegX,VEMP);break;			/* CMPX #$ */
	case 0x8d: *C=7;Dummy=RegPC++;Pshs(CPU,0x80);RegPC+=Rd8(Dummy);break;	/* BSR */
	case 0x8e: *C=3;Tstw(CPU,RegX=MGETE);break;              		/* LDX  #$ */

	case 0x90: *C=4;Subc(CPU,&RegA,Rd8(MGETD));break;			/* SUBA /$ */
	case 0x91: *C=4;Cmpc(CPU,RegA,Rd8(MGETD));break;			/* CMPA /$ */
	case 0x92: *C=4;Sbc (CPU,&RegA,Rd8(MGETD));break;			/* SBCA /$ */
	case 0x93: *C=6;Subw(CPU,&RegD,Rd16(MGETD));break;			/* SUBD /$ */
	case 0x94: *C=4;Tstc(CPU,RegA &=Rd8(MGETD));break;			/* ANDA /$ */
	case 0x95: *C=4;Tstc(CPU,RegA & Rd8(MGETD));break;			/* BITA /$ */
	case 0x96: *C=4;Tstc(CPU,RegA = Rd8(MGETD));break;			/* LDA /$  */	
	case 0x97: *C=4;Wr8(MGETD,RegA);Tstc (CPU,RegA);break;			/* STA /$  */	
	case 0x98: *C=4;Tstc(CPU,RegA ^=Rd8(MGETD));break;			/* EORA /$ */	
	case 0x99: *C=4;Adc (CPU,&RegA,Rd8(MGETD));break;			/* ADCA /$ */
	case 0x9A: *C=4;Tstc(CPU,RegA |=Rd8(MGETD));break;			/* ORA /$  */
	case 0x9B: *C=4;Addc(CPU,&RegA,Rd8(MGETD));break;			/* ADDA /$ */
	case 0x9C: *C=6;Cmpw(CPU,RegX,Rd16(MGETD));break;			/* CMPX /$ */
	case 0x9D: *C=7;MGETD;Pshs(CPU,0x80);RegPC=VDPR;break;			/* JSR /$  */
	case 0x9E: *C=5;Tstw (CPU,RegX=Rd16(MGETD));break;			/* LDX /$  */
	case 0x9F: *C=5;Wr16 (MGETD,RegX);Tstw(CPU,RegX);break;			/* STX /$  */

  	case 0xa0: *C=4;Subc(CPU,&RegA,Rd8(MGETI)); break;                    	/* SUBA IX */
  	case 0xa1: *C=4;Cmpc(CPU,RegA,Rd8(MGETI)); break;              	      	/* CMPA IX */
  	case 0xa2: *C=4;Sbc(CPU,&RegA,Rd8(MGETI)); break;              		/* SBCA IX */
  	case 0xa3: *C=6;Subw(CPU,&RegD,Rd16(MGETI)); break;                   	/* SUBD IX */
  	case 0xa4: *C=4;Tstc(CPU,RegA &= Rd8(MGETI)); break;                 	/* ANDA IX */
  	case 0xa5: *C=4;Tstc(CPU,Rd8(MGETI) & RegA); break;                  	/* BITA IX */
  	case 0xa6: *C=4;Tstc (CPU,RegA=Rd8(MGETI));break;			/* LDA IX*/
  	case 0xa7: *C=4;Wr8(MGETI,RegA); Tstc(CPU,RegA); break;                	/* STA  IX */
  	case 0xa8: *C=4;Tstc(CPU,RegA ^= Rd8(MGETI)); break;                 	/* EORA IX */
  	case 0xa9: *C=4;Adc(CPU,&RegA,Rd8(MGETI)); break;                     	/* ADCA IX */
  	case 0xaa: *C=4;Tstc(CPU,RegA |= Rd8(MGETI)); break;                 	/* ORA  IX */
  	case 0xab: *C=4;Addc(CPU,&RegA, Rd8(MGETI)); break;                    	/* ADDA IX */
  	case 0xac: *C=6;Cmpw(CPU,RegX, Rd16(MGETI)); break;                    	/* CMPX IX */
  	case 0xad: *C=7;Dummy=MGETI;Pshs(CPU,0x80);RegPC=Dummy; break;         	/* JSR  IX */
  	case 0xae: *C=5;Tstw(CPU,RegX=Rd16(MGETI)); break;                   	/* LDX  IX */
  	case 0xaf: *C=5;Wr16(MGETI,RegX);Tstw(CPU,RegX); break;                	/* STX  IX */

	case 0xb0: *C=5;Subc(CPU,&RegA,Rd8(MGETE));break; 			/* SUBA $  */
	case 0xb1: *C=5;Cmpc(CPU,RegA,Rd8(MGETE));break; 			/* CMPA $  */
	case 0xb2: *C=5;Sbc(CPU,&RegA,Rd8(MGETE));break; 			/* SBCA $  */
	case 0xb3: *C=7;Subw(CPU,&RegD,Rd16(MGETE));break;			/* SUBD $  */
	case 0xb4: *C=5;Tstc(CPU,RegA &=Rd8(MGETE));break;			/* ANDA $  */
	case 0xb5: *C=5;Tstc(CPU,RegA & Rd8(MGETE));break;			/* BITA $  */
	case 0xb6: *C=5;Tstc(CPU,RegA=Rd8(MGETE));break; 			/* LDA $   */
	case 0xb7: *C=5;Wr8(MGETE,RegA);Tstc (CPU,RegA);break;			/* STA $   */
	case 0xb8: *C=5;Tstc(CPU,RegA ^=Rd8(MGETE));break;			/* EORA $  */
	case 0xb9: *C=5;Adc(CPU,&RegA,Rd8(MGETE));break;			/* ADCA $  */
	case 0xba: *C=5;Tstc(CPU,RegA |=Rd8(MGETE));break;			/* ORA $   */
	case 0xbb: *C=5;Addc(CPU,&RegA,Rd8(MGETE));break;			/* ADDA $  */
	case 0xbc: *C=7;Cmpw(CPU,RegX,Rd16(MGETE));break;			/* CMPX $  */
	case 0xbd: *C=8;MGETE;Pshs(CPU,0x80);RegPC=VEMP;break;			/* JSR $   */
	case 0xbe: *C=6;Tstw(CPU,RegX=Rd16(MGETE));break;			/* LDX $   */
	case 0xbf: *C=6;Wr16(MGETE,RegX);Tstw(CPU,RegX);break;			/* STX $   */
	
	case 0xc0: *C=2;Subc (CPU,&RegB,Rd8 (RegPC++));break;			/* SUBB #$ */	
	case 0xc1: *C=2;Cmpc(CPU,RegB, Rd8(RegPC++)); break; 	                /* CMPB #$ */
	case 0xc2: *C=2;Sbc (CPU,&RegB,Rd8 (RegPC++));break;			/* SBCB #$ */		
	case 0xc3: *C=4;Addw(CPU,&RegD,MGETE);break;				/* ADDD #$ */
	case 0xc4: *C=2;Tstc(CPU,RegB &=Rd8(RegPC++));break;			/* ANDB #$ */
	case 0xc5: *C=2;Tstc(CPU,RegB & Rd8(RegPC++));break;			/* BITB #$ */	
	case 0xc6: *C=2;Tstc(CPU,RegB=Rd8(RegPC++));break;	        	/* LDB  #$ */
	case 0xc8: *C=2;Tstc(CPU,RegB ^=Rd8(RegPC++));break;			/* EORB #$ */	
	case 0xc9: *C=2;Adc (CPU,&RegB,Rd8(RegPC++));break;			/* ADCb #$ */	
	case 0xca: *C=2;Tstc(CPU,RegB |=Rd8(RegPC++));break;			/* ORB #$  */		
	case 0xcb: *C=2;Addc(CPU,&RegB,Rd8(RegPC++));break;			/* ADDB #$ */	
	case 0xcc: *C=3;Tstw(CPU,RegD=MGETE);break;              		/* LDD  #$ */
	case 0xce: *C=3;Tstw(CPU,RegU=MGETE);break;              		/* LDU  #$ */

	case 0xD0: *C=4;Subc(CPU,&RegB,Rd8(MGETD));break;			/* SUBB /$ */
	case 0xD1: *C=4;Cmpc(CPU,RegB,Rd8(MGETD));break;			/* CMPB /$ */
	case 0xD2: *C=4;Sbc (CPU,&RegB,Rd8(MGETD));break;			/* SBCB /$ */
	case 0xD3: *C=6;Addw(CPU,&RegD,Rd16(MGETD));break;			/* ADDD /$ */
	case 0xD4: *C=4;Tstc(CPU,RegB &=Rd8(MGETD));break;			/* ANDB /$ */
	case 0xD5: *C=4;Tstc(CPU,RegB & Rd8(MGETD));break;			/* BITB /$ */
	case 0xD6: *C=4;Tstc(CPU,RegB = Rd8(MGETD));break;			/* LDB /$  */	
	case 0xD7: *C=4;Wr8(MGETD,RegB);Tstc (CPU,RegA);break;			/* STB /$  */	
	case 0xD8: *C=4;Tstc(CPU,RegB ^=Rd8(MGETD));break;			/* EORB /$ */	
	case 0xD9: *C=4;Adc (CPU,&RegB,Rd8(MGETD));break;			/* ADCB /$ */
	case 0xDA: *C=4;Tstc(CPU,RegB |=Rd8(MGETD));break;			/* ORB /$  */
	case 0xDB: *C=4;Addc(CPU,&RegB,Rd8(MGETD));break;			/* ADDB /$ */
	case 0xDC: *C=5;Tstw(CPU,RegD=Rd16(MGETD));break;			/* LDD  /$ */
	case 0xDD: *C=5;Wr16 (MGETD,RegD);Tstw(CPU,RegD);break;			/* STD  /$ */
	case 0xDE: *C=5;Tstw(CPU,RegU=Rd16(MGETD));break;			/* LDU  /$ */	
	case 0xDF: *C=5;Wr16 (MGETD,RegU);Tstw(CPU,RegU);break;			/* STU  /$ */
	
	case 0xe0: *C=4;Subc(CPU,&RegB,Rd8(MGETI));break;			/* SUBB IX */
	case 0xe1: *C=4;Cmpc(CPU,RegB,Rd8(MGETI));break;			/* CMPB IX */
	case 0xe2: *C=4;Sbc (CPU,&RegB,Rd8(MGETI));break;			/* SBCB IX */
	case 0xe3: *C=6;Addw(CPU,&RegD,Rd16(MGETI));break;			/* ADDD IX */
	case 0xe4: *C=4;Tstc(CPU,RegB &=Rd8(MGETI));break;			/* ANDB IX */
	case 0xe5: *C=4;Tstc(CPU,Rd8(MGETI) & RegB);break;			/* BITB IX */
	case 0xe6: *C=4;Tstc(CPU,RegB = Rd8(MGETI));break;			/* LDB IX  */
	case 0xe7: *C=4;Wr8(MGETI,RegB);Tstc(CPU,RegB);break;			/* STB IX  */
	case 0xe8: *C=4;Tstc(CPU,RegB ^=Rd8(MGETI));break;			/* EORB IX */
	case 0xe9: *C=4;Adc(CPU,&RegB,Rd8(MGETI));break;			/* ADCB IX */
	case 0xea: *C=4;Tstc(CPU,RegB |=Rd8(MGETI));break;			/* ORB IX */
  	case 0xeb: *C=4;Addc(CPU,&RegB, Rd8(MGETI)); break;                    	/* ADDB IX */
  	case 0xec: *C=5;Tstw(CPU, RegD = Rd16(MGETI)); break;                   /* LDD  IX */
  	case 0xed: *C=5;Wr16(MGETI,RegD);Tstw(CPU,RegD);break; /*Added Tstw */	/* STD  IX */
  	case 0xee: *C=5;Tstw(CPU,RegU = Rd16(MGETI)); break;                   	/* LDU  IX */
  	case 0xef: *C=5;Wr16(MGETI,RegU); Tstw(CPU,RegU); break;                /* STU  IX */
	
	case 0xf0: *C=5;Subc(CPU,&RegB,Rd8(MGETE));break; 			/* SUBB $  */
	case 0xf1: *C=5;Cmpc(CPU,RegB,Rd8(MGETE));break; 			/* CMPB $  */
	case 0xf2: *C=5;Sbc(CPU,&RegB,Rd8(MGETE));break; 			/* SBCB $  */
	case 0xf3: *C=7;Addw(CPU,&RegD,Rd16(MGETE));break;			/* ADDD $  */
	case 0xf4: *C=5;Tstc(CPU,RegB &=Rd8(MGETE));break;			/* ANDB $  */
	case 0xf5: *C=5;Tstc(CPU,RegB & Rd8(MGETE));break;			/* BITB $  */
	case 0xf6: *C=5;Tstc(CPU,RegB=Rd8(MGETE));break; 			/* LDB $   */
	case 0xf7: *C=5;Wr8(MGETE,RegB);Tstc (CPU,RegB);break;			/* STB $   */
	case 0xf8: *C=5;Tstc(CPU,RegB ^=Rd8(MGETE));break;			/* EORB $  */
	case 0xf9: *C=5;Adc(CPU,&RegB,Rd8(MGETE));break;			/* ADCB $  */
	case 0xfa: *C=5;Tstc(CPU,RegB |=Rd8(MGETE));break;			/* ORB $   */
	case 0xfb: *C=5;Addc(CPU,&RegB,Rd8(MGETE));break;			/* ADDB $  */
	case 0xfc: *C=6;Tstw(CPU,RegD=Rd16(MGETE));break;			/* LDD $   */
	case 0xfd: *C=6;Wr16(MGETE,RegD);Tstw(CPU,RegD);break;	/*Added Tstw */	/* STD $   */
	case 0xfe: *C=6;Tstw (CPU,RegU=Rd16(MGETE));break;			/* LDU $   */
	case 0xff: *C=6;Wr16(MGETE,RegU);Tstw(CPU,RegU);break;			/* STU $   */
	default: r=1;*C=1;break;
	}
CPU->Cycles+=*C; /* Update Cycle counter */
return (r);
}

/*----------------------------------------------------------------------*/
/* Memory Access: Mgetd Direct page					*/
/*----------------------------------------------------------------------*/

/* Can be call with MGETD Macro*/
s16 Mgetd(M6809CPU *CPU)
{
RegDD = Rd8(RegPC++);	/* CPU Low DPR is filled with imm8 */
return VDPR;		/* Return CPU DPR, but result is store to in CPU->DPR */
}

/* Can be call with MGETE Macro*/
s16 Mgete(M6809CPU *CPU)
{
VEMPh=Rd8(RegPC++);	/* Fill VEMP with 2 bytes after opcode. Can be use imm16 value */
VEMPl=Rd8(RegPC++);	/* or extend (16b) ptr. PC is incremented twice. */
return VEMP;
}

/* Can be call with MGETI Macro*/
s16 Mgeti(M6809CPU *CPU,u8 *C)
{
u8 c=Rd8(RegPC++);			/* Read Indexed adressing mode post byte */
u16 *rx = CPU->pReg16[((c&0x60)>>5)]; 	/* rx points to X,Y,U or S register*/
switch (c &= 0x9f)
	{
	case 0x80: *C+=2;*rx +=1; return (*rx-1);	/* C+=2; ,R+ R value is used as memory index then increased*/
	case 0x81: *C+=3;*rx +=2; return (*rx-2);	/* C+=3; ,R++ = .... then add 2 */  
	case 0x82: *C+=2;return (*rx -=1);		/* C+=2; ,-R	 */
	case 0x83: *C+=3;return (*rx -=2);		/* C+=3; ,--R	 */
	case 0x84: return (*rx);			/* C+=0; ,R	 */
	case 0x85: *C+=1;return(*rx + RegB);          	/* C+=1; B,R (B+R)value is used as memory index  */
	case 0x86: *C+=1;return(*rx + RegA);          	/* C+=1; A,R   */
	case 0x88: *C+=1;return(*rx + Rd8(RegPC++));	/* C+=1; +/- 7bit value + R */
	case 0x89: *C+=4;return(*rx + MGETE);		/* C+=4; +/- 15bit value + R */
	case 0x8b: *C+=4;return(*rx + RegD);		/* C+=4; D+R is used as mem index */
	case 0x8c: *C+=1;RegPC++;return (RegPC+Rd8(RegPC-1)); /* C+=1; +/-7bit + PC */
	case 0x8d: *C+=5;MGETE;return (RegPC+VEMP);	/* C+=5; +/- 15bit + PC */
	case 0x91: *C+=3;*rx+=2;return (Rd16(*rx-2));	/* C+=3; [,R++] = RAM[RAM [rx++ ++]] */
	case 0x93: *C+=3;*rx-=2;return (Rd16(*rx));	/* C+=3; [,--R] = RAM[RAM [-- --rx]] */	
	case 0x94: return (Rd16(*rx));			/* C+=0; [,R] = RAM[RAM [rx]] */	
	case 0x95: *C+=1;return (Rd16(*rx+RegB));	/* C+=1; [B,R] = RAM[RAM [B+rx]] */		
	case 0x96: *C+=1;return (Rd16(*rx+RegA));	/* C+=1; [A,R] = RAM[RAM [A+rx]] */	
    	case 0x98: *C+=1;return(Rd16(*rx+Rd8(RegPC++))); 	/* C+=1; RAM[RAM[s8 + R]]   */
  	case 0x99: *C+=4;return(Rd16(*rx+MGETE));            	/* C+=4; RAM[RAM[s16 + R]]  */
  	case 0x9b: *C+=4;return(Rd16(*rx+RegD));        		/* C+=4; RAM[RAM[D,R]]      */
  	case 0x9c: *C+=1;RegPC++;return(Rd16(RegPC+Rd8(RegPC-1))); 	/* C+=1; RAM[RAM[s8+PCR]]*/
  	case 0x9d: *C+=5;MGETE;return(Rd16(RegPC + VEMP));         	/* C+=5; RAM[RAM[s16+PCR]]*/
  	case 0x9f: *C+=5;return(Rd16(MGETE));                      	/* C+=5; RAM[RAM[s16]]    */
  	default:
  	  /*
	  All other case treat as 5 bit,R
	  Assumes 0RRnnnnn for illegal bytes
	  0x87 0x8a 0x8e 0x8f 0x90 0x92 0x97 0x9a 0x9e
	  */
  	if(c & 0x10) c |= 0xf0; 
  	return(*rx + (s8)c);      /* C+=1;  5 bits,R  */
	/*break; Unreachable because after a return */
	}
}

/*----------------------------------------------------------------------*/
/* Tstc/Tstw Test instruction. (CC=EFHINZVC)				*/
/* Compute instruction and set cpu CC flags.				*/
/*----------------------------------------------------------------------*/
void Tstc(M6809CPU *CPU,s8 c)
{
 RegCC &= 0xf1;					/* CC = ????000? */
 if(c & 0x80) RegCC |= NF;			/* tst bit sign, set NF if needed */
 if(c == 0) RegCC |= ZF;			/* Set zero flag if c null */
}

void Tstw(M6809CPU *CPU,s16 w)
{
 RegCC &= 0xf1;					/* CC = ????000? */
 if(w & 0x8000) RegCC |= NF;			/* tst bit sign, set NF if needed */
 if(w == 0) RegCC |= ZF;			/* Set zero flag if c null */
}
/*----------------------------------------------------------------------*/
/* Cmpw/Cmpc compare instruction. (CC=EFHINZVC)				*/
/* Compute instruction and set cpu CC flags.				*/
/*----------------------------------------------------------------------*/
void Cmpc(M6809CPU *CPU,s8 reg,s8 c)
{
 char r = reg;
 int i = r - c;
 RegCC &= 0xf0;					/* CC = ????0000 */
 if(((u8)r - (u8)c) & 0x100) RegCC |= CF;	/* Set Carry if reg<c */
 r = i & 0xff;					/* r = (r-c) & 0xff */
 if(r & 0x80) RegCC |= NF;			/* Set Negative flag if needed */
 if(r == 0) RegCC |= ZF;			/* Set Zero flag if reg=c (r-c=0) */
 if(r != i) RegCC |= VF;			/* r != (r-c) -> overflow */
}

void Cmpw(M6809CPU *CPU,s16 reg,s16 w)
{
 short r = reg;
 long i = reg - w;
 RegCC &= 0xf0;					/* CC = ????0000 */
 if(((u16)r - (u16)w) & 0x10000) RegCC |= CF;	/* Set Carry if reg<c */
 r = i & 0xffff;				/* r = (r-c) & 0xff */
 if(r & 0x8000) RegCC |= NF;			/* Set Negative flag if needed */
 if(r == 0) RegCC |= ZF;			/* Set Zero flag if reg=c (r-c=0) */
 if(r != i) RegCC |= VF;			/* r != (r-c) -> overflow */
}

/*----------------------------------------------------------------------*/
/*   NEG, COM, CLR, INC, DEC instruction. (CC=EFHINZVC)			*/
/* Compute instruction and set cpu CC flags.				*/
/*----------------------------------------------------------------------*/
s8 Neg(M6809CPU *CPU, s8 c)
{
 c = 0x100 - c;
 RegCC &= 0xf0;			/* CC = ????0000 */
 if(c & 0x80) RegCC |= NF;	/* Set Negative flag if Neg(C)<0 */
 if(c == 0) RegCC |= ZF;	/* Set Zero if Neg(C)=0 */
 if(c == -0x80) RegCC |= VF;	/* Set Overflow if needed */
 if(c != 0) RegCC |= CF;	/* Set Carry On */
 return(c);
}

s8 Com(M6809CPU *CPU,s8 c)
{
c = ~c;
RegCC &= 0xf0;			/* CC = ????0000 */
if(c & 0x80) RegCC |= NF;	/* Set Negative flag if C<0 */
if(c == 0) RegCC |= ZF;		/* Set Zero flag if C = 0 */
RegCC |= CF;			/* CC = ????*/
return(c);
}

s8 Inc(M6809CPU *CPU,s8 c)
{
RegCC &= 0xf1;			/* CC = ????000? */
if(++c & 0x80) RegCC |= NF;	/* Set Negative flag if C<0 */
if(c == 0) RegCC |= ZF;		/* Set Zero if C=0 */
if(c == -0x80) RegCC |= VF;	/* Set Overflow if needed. if c=-80h we have made a 8bit warp*/
return(c);
}

s8 Dec(M6809CPU *CPU,s8 c)
{
RegCC &= 0xf1;			/* CC = ????000? */
if(--c & 0x80) RegCC |= NF;	/* Set Negative flag if C<0 */
if(c == 0) RegCC |= ZF;		/* Set Zero if C=0 */
if(c == 0x7f) RegCC |= VF;	/* Set Overflow if needed */
return(c);
}

s8 Clr(M6809CPU *CPU)
{
RegCC &= 0xf0;			/* CC = ????0000 */
RegCC |= ZF;			/* CC = ????0100 */
return(0);
}
/*----------------------------------------------------------------------*/
/*   Lsr, Ror, Rol, Asr, Asl Instructions (CC=EFHINZVC)			*/
/* Compute instruction and set cpu CC flags.				*/
/*----------------------------------------------------------------------*/
s8 Lsr(M6809CPU *CPU,u8 c)	/* less significative bit goes in Carry */
{				/* Logical shift right */
RegCC &= 0xf2;			/* CC = ????00?0*/
if(c < 0x02) RegCC |= ZF;	/* C will be null after shift: Set ZF*/
if(c & 0x01) RegCC |= CF;	/* Bit0 go 'out': set Carry*/
return(c >> 1);
}

s8 Ror(M6809CPU *CPU,u8 c)
{				/* Rotate Right */
s8 r = (c >> 1) | (RegCC << 7); /* C>>=1 Carry goes in c most signif. bit */
RegCC &= 0xf2;			/* CC = ????00?0*/
if(r & 0x80) RegCC |= NF;	/* Set Negative flag if needed */
if(r == 0) RegCC |= ZF;		/* Set Zero flag if r = 0 */
if(c & 0x01) RegCC |= CF;	/* going out bit, less significative bit goes in Carry */
return(r);
}

s8 Rol(M6809CPU *CPU,u8 c)
{				/* Rotate left */
s8 r = (c << 1) | (RegCC & CF); /* C<<=1 Carry goes in less significative bit */
RegCC &= 0xf0;			/* CC = ????0000 */
if(r & 0x80) RegCC |= NF;	/* Set Negative flag if needed */
if(r == 0) RegCC |= ZF;		/* Set Zero flag if r = 0 */
if((r ^ c) & 0x80) RegCC |= VF; /* Set Overflow if needed */
if(c & 0x80) RegCC|= CF;	/* bit7 going out, most significative bit goes in Carry */
return(r);
}

s8 Asr(M6809CPU *CPU,u8 c)
{				/* Arithmetic Shift Right */
s8 r = (c >> 1) | (c & 0x80);	/* C>>=1 Save sign bit ?*/
RegCC &= 0xf2;			/* CC = ????00?0 */
if(r & 0x80) RegCC |= NF;	/* Set negative flag if r negative */
if(r == 0) RegCC |= ZF;		/* Set zero flag if r = zero */
if(c & 0x01) RegCC |= CF;	/* Bit0 going out, goes in Carry */
return(r);
}

s8 Asl(M6809CPU *CPU,u8 c)
{				/* Arithmetic Shift left */
char r = c << 1;		/* c <<=1 */
RegCC &= 0xf0;			/* CC = ????0000 */
if(r & 0x80) RegCC |= NF;	/* Set negative flag if r negative */
if(r == 0) RegCC |= ZF;		/* Set zero flag if r = zero */
if((r ^ c) & 0x80) RegCC |= VF;	/* Set Overflow if needed */
if(c & 0x80) RegCC|= CF;	/* Bit7 going out, goes in Carry */
return(r);
}

/*----------------------------------------------------------------------*/
/*   Mul, Adc, Addc,Adw, Subc, Sbc, Subw, Daa Instructions (CC=EFHINZVC)*/
/* Compute instruction and set cpu CC flags.				*/
/*----------------------------------------------------------------------*/

void Mul (M6809CPU *CPU)
{				/* Multiplication of A and B reg. Result is stored in D */
RegD = ((u8)RegA) * ((u8)RegB); /* D = A*B */
RegCC &= 0xf2;			/* CC = ????00?0 */
if(RegD == 0) RegCC |= ZF;	/* Set Zero flag if needed */
if(RegD & 0x80) RegCC |= CF;	/* Set Carry*/
}

void Addc(M6809CPU *CPU,s8 *reg, s8 c)
{							/* Add char without carry */
int i = *reg + c;					/* dst reg += c*/
RegCC &= 0xd0;						/* CC = ??0?0000 */
if(((*reg & 0x0f) + (c & 0x0f)) & 0x10) RegCC |= HF;	/* if 4 bit add & 0x10 set HalfFlag ???*/
if(((u8)*reg + (u8)c) & 0x100) RegCC |= CF;		/* Set carry*/
*reg = i & 0xff;					/* reg = 8bit value*/
if(*reg & 0x80) RegCC |= NF;				/* Set negative flag if needed*/
if(*reg == 0) RegCC |= ZF;				/* Set Zero flag if needed */
if(*reg != i) RegCC |= VF;				/* Check for overflow */
}

void Adc(M6809CPU *CPU,s8 *reg, s8 c)
{							/* Add char with carry */
int carry = (RegCC & CF);				/* carry =0 or 1 dep on carry */
int i = *reg + c + carry;				/* add with carry */
RegCC &= 0xd0;						/* CC = ??0?0000*/
if(((*reg & 0x0f) + (c & 0x0f) + carry) & 0x10) RegCC |= HF; /* ??? */
if(((u8)*reg + (u8)c + carry) & 0x100) RegCC |= CF;	/* Set new carry if needed*/
*reg = i & 0xff;					/* Wrap around 8bit value (%256)*/
if(*reg & 0x80) RegCC |= NF;				/* Set negative flag if needed */
if(*reg == 0) RegCC |= ZF;				/* Set Zero flag if needed */
if(*reg != i) RegCC |= VF;				/* check for overflow */
}

void Addw(M6809CPU *CPU,s16 *reg, s16 w)
{						/* Add without carry */
long i = *reg + w;				/* add */
RegCC &= 0xf0;					/* CF = ????0000 */
if(((u16)*reg + (u16)w) & 0x10000) RegCC |= CF;	/* Set carry if needed */
*reg = i & 0xffff;				/* Wrap around 16bit value */
if(*reg & 0x8000) RegCC |= NF;			/* Set negative flag if needed */
if(*reg == 0) RegCC |= ZF;			/* Set Zero flag if needed*/
if(*reg != i) RegCC |= VF;			/* Set Overflow flag if needed */
}

void Subc(M6809CPU *CPU,s8 *reg, s8 c)
{						/* Sub without carry */
int i = *reg - c;				/* reg-=c */
RegCC &= 0xf0;					/* CC = ????0000 */
if(((u8)*reg - (u8)c) & 0x100) RegCC |= CF;	/* Set carry if nedeed*/
*reg = i & 0xff;				/* Wrap around 8bit value */ 
if(*reg & 0x80) RegCC |= NF;			/* Set negative flag if needed */
if(*reg == 0) RegCC |= ZF;			/* Set Zero flag if needed */
if(*reg != i) RegCC |= VF;			/* check for overflow */
}

void Sbc(M6809CPU *CPU,s8 *reg, s8 c)
{						/* Sub with carry */
int carry = (RegCC & CF);			/* carry = 0 or 1 dep. on carry */
int i = *reg - c - carry;			/* sub with carry */
RegCC &= 0xf0;					/* CC = ????0000 */
if(((u8)*reg - (u8)c - carry) & 0x100) RegCC |= CF;
*reg = i & 0xff;				/* Wrap around 8bit value */ 
if(*reg & 0x80) RegCC |= NF;			/* Set negative flag if needed */
if(*reg == 0) RegCC |= ZF;			/* Set Zero flag if needed */
if(*reg != i) RegCC |= VF;			/* check for overflow */;
}

void Subw(M6809CPU *CPU,s16 *reg, s16 w)
{						/* sub word without carry */
long i = *reg - w;				/* i=reg-w*/
RegCC &= 0xf0;					/* CC = ????0000 */
if(((u16)*reg - (u16)w) & 0x10000) RegCC |= CF;	/* Set carry if needed */
*reg = i & 0xffff;				/* Wrap around 16bit value */
if(*reg & 0x8000) RegCC |= NF;			/* Test and set neg. flag if needed */
if(*reg == 0) RegCC |= ZF;			/* Test and set Zero flag if needed */
if(*reg != i) RegCC |= VF;			/* Test and set overflow flag if needed */
}

s8 Daa(M6809CPU *CPU,u8 c)
{						/* Decimal Adjustment Accumulator*/
s8 r;
int i = c;
if((RegCC & HF) || ((i & 0x00f) > 0x09)) i += 0x06;/* A->0, B->1,C->2... for c 4 lower bits*/
if((RegCC & CF) || ((i & 0x1f0) > 0x90)) i += 0x60;/* A->0, B->1,C->2... for c 4 higher bits*/
r = i & 0xff;					/* Warp around 8bit value */
c = (i >> 1) | (RegCC << 7);			/* c/=2 and Set carry in C's bit7 */
RegCC &= 0xf0;					/* CC = ????0000 */
if(r & 0x80) RegCC |= NF;			/* Test and set neg. flag if needed */
if(r == 0) RegCC |= ZF;				/* Test and set zero flag if needed */
if((r ^ c) & 0x80) RegCC |= VF;			/* Test and set overflow flag if needed */
if(c & 0x80) RegCC |= CF;			/* Test and set carry flag if needed */
return(r);
}

/*----------------------------------------------------------------------*/
/*   Puls,Pulu,Pshs,Pshu Stack Instructions (CC=EFHINZVC)		*/
/* Compute stack instruction .						*/
/*----------------------------------------------------------------------*/
void Pshu (M6809CPU *CPU,u8 c)
{
if(c & 0x80) {Wr8(--RegU,PCl);Wr8(--RegU,PCh);}
if(c & 0x40) {Wr8(--RegU,Sl); Wr8(--RegU,Sh);}	/* Push S not U !*/
if(c & 0x20) {Wr8(--RegU,Yl); Wr8(--RegU,Yh);}
if(c & 0x10) {Wr8(--RegU,Xl); Wr8(--RegU,Xh);}
if(c & 0x08) {Wr8(--RegU,RegDP);}
if(c & 0x04) {Wr8(--RegU,RegB);}
if(c & 0x02) {Wr8(--RegU,RegA);}
if(c & 0x01) {Wr8(--RegU,RegCC);}
}

void Pulu (M6809CPU *CPU,u8 c)
{
if(c & 0x01) {RegCC = Rd8(RegU); RegU++;}
if(c & 0x02) {RegA =  Rd8(RegU); RegU++;}
if(c & 0x04) {RegB =  Rd8(RegU); RegU++;}
if(c & 0x08) {RegDP = Rd8(RegU); RegU++;}
if(c & 0x10) {Xh = Rd8(RegU); RegU++; Xl=Rd8(RegU); RegU++;}
if(c & 0x20) {Yh = Rd8(RegU); RegU++; Yl=Rd8(RegU); RegU++;}
if(c & 0x40) {Sh = Rd8(RegU); RegU++; Sl=Rd8(RegU); RegU++;}
if(c & 0x80) {PCh = Rd8(RegU); RegU++; PCl=Rd8(RegU); RegU++;}
}

void Pshs (M6809CPU *CPU,u8 c)
{
if(c & 0x80) {Wr8(--RegS,PCl);Wr8(--RegS,PCh);}
if(c & 0x40) {Wr8(--RegS,Ul); Wr8(--RegS,Uh);}	/* Push U not S !*/
if(c & 0x20) {Wr8(--RegS,Yl); Wr8(--RegS,Yh);}
if(c & 0x10) {Wr8(--RegS,Xl); Wr8(--RegS,Xh);}
if(c & 0x08) {Wr8(--RegS,RegDP);}
if(c & 0x04) {Wr8(--RegS,RegB);}
if(c & 0x02) {Wr8(--RegS,RegA);}
if(c & 0x01) {Wr8(--RegS,RegCC);}
}

void Puls (M6809CPU *CPU,u8 c)
{
if(c & 0x01) {RegCC = Rd8(RegS); RegS++;}
if(c & 0x02) {RegA =  Rd8(RegS); RegS++;}
if(c & 0x04) {RegB =  Rd8(RegS); RegS++;}
if(c & 0x08) {RegDP = Rd8(RegS); RegS++;}
if(c & 0x10) {Xh = Rd8(RegS); RegS++; Xl=Rd8(RegS); RegS++;}
if(c & 0x20) {Yh = Rd8(RegS); RegS++; Yl=Rd8(RegS); RegS++;}
if(c & 0x40) {Uh = Rd8(RegS); RegS++; Ul=Rd8(RegS); RegS++;}
if(c & 0x80) {PCh = Rd8(RegS);RegS++; PCl=Rd8(RegS);RegS++;}
}


/*----------------------------------------------------------------------*/
/*   Exg,Tfr Instructions 						*/
/* RgPtr[x] x = 6,7,12,13,14,15 are invalid register. They are 		*/
/* redirected in cpu Dum register.
/*----------------------------------------------------------------------*/
void Exg(M6809CPU *CPU,u8 c)
{
u8 c1 = c>>4;		/* 4 upper bit for operand 1 */
u8 c2 = c&0x0f;		/* 4 lower bit for operand 2 */
if(c1 < 6)		/* Mean a 16 bit register: D,X,Y,U,S or PC */
	{
	s16 x = *(s16 *)RgPtr[c2];	/* x is a temporary variable */
	*(s16 *)RgPtr[c2] = *(s16 *)RgPtr[c1]; /* Exchange registers  */
	*(s16 *)RgPtr[c1] = x;
	}
else	/* Mean a 8 bit register: Dum,Dum,A,B,CC,DP,Dum,Dum,Dum,Dum*/
	{
	s8 x = *(s8 *)RgPtr[c2];
	*(s8 *)RgPtr[c2] = *(s8 *)RgPtr[c1];
	*(s8 *)RgPtr[c1] = x;
	}
}

void Tfr(M6809CPU *CPU,u8 c)
{
u8 c1 = c>>4;		/* 4 upper bit for operand 1 */
u8 c2 = c&0x0f;		/* 4 lower bit for operand 2 */
if(c1 < 6) *(s16 *)RgPtr[c2] = *(s16 *)RgPtr[c1];	/* 16 bit var transfert */
else *(s8 *)RgPtr[c2] = *(s8 *)RgPtr[c1];		/* 8 bit var transfert */
}





/*----------------------------------------------------------------------*/
/*   Irq, Firq instructions.						*/
/* 									*/
/*----------------------------------------------------------------------*/
void Irq (M6809CPU *CPU,u16 addr)
{
RegCC |= EF;		/* Set flag*/
Pshs(CPU,0xff);		/* Push all registers on stack S */
/* CC |= FI1; ?????? */
RegCC |= INF;		/* Set Interrupt flag */
PCh = Rd8(addr);	/* PC = $addr */
PCl = Rd8(addr+1);
}

void Firq (M6809CPU *CPU,u16 addr)
{
RegCC &= EF0;		/* Clear CC's E bit */
Pshs(CPU,0x81);		/* Push PC,CC on Stack S */
RegCC |= FF;		/* Set FastIrq bit in CC */
RegCC |= INF;		/* Set Interrupt flag */
PCh = Rd8(addr);	/* PC = $addr */
PCl = Rd8(addr+1);
}