/*--------------------------------------------------------------*/
/* Mikie (c)1984 Konami, arcade emulator.			*/
/* Written by Bruno VEDDER. 6809 Cpu emulation based on 	*/
/* Daniel Coulomb work (http://dcmo5.free.fr/), and modified for*/
/* Multi-Cpu and accurate cycles counting.			*/
/*--------------------------------------------------------------*/
/* Special thanks to Sean Riddle for helping me with color	*/
/* decoding, and mame team for their unvaluable sources.	*/
/*--------------------------------------------------------------*/
/* This source is provided as is, without warranty. Feel free   */
/* to use this code, and modify it for non profit/commercial	*/
/* purpose. If you have any questions about this program don't	*/
/* hesitate and drop me a mail at bruno.vedder@emu-france.com	*/
/* Note: Don't ask for ROM, they won't be provided !!!		*/
/*______________________________________________________________*/
/*								*/
/* ---[Bruno's Coding Zone]--------------------------		*/
/*    URL http://bcz.emu-france.com/  				*/
/*    Mail: bruno.vedder@emu-france.com				*/
/*______________________________________________________________*/
/*								*/

#include "SRC\SDL.h"   /* All SDL App's need this */
#include <stdio.h>
#include <stdlib.h>
#include <mem.h>
#include "6809.h"
#include "6809emu.h"

void DisplayInit (void);
void Mikie_InitVideoHardWare (void);
void UpdateVideo (void);
void SDL_React (void);
u32 TimerHandler (void);

void AllocBuffer (u8 **var,u32 size,char *name);
void DrawChar8x8Bloc (s32 x,s32 y,u32 nbr,u8 CM);
void LoadRom (u8 *dst,u8 *filename,s32 size);
void DrawSprite (s32 x,s32 y,u16 n,u8 p);
void DrawSprites (void);
void DrawXinvSprite (s32 x,s32 y,u16 n,u8 p);
void Mikie_ConvertGfxRoms (void);
void Rotate16x16gfx (u16 n);
u8   GetBit (u32 bit_nbr,u8 *buffer);

u8  RefreshFlag=0;		/* 30 Hertz  flag 	  */
u8  ToggleFlag =0;		/* Refresh 1/2 		  */
u32 TFrame =0;			/* Total displayed frames */
/*----------------------------------------------------------------------*/
/* Delay between 2 frame. 						*/
/* Due to PC timer imprecision, we use a 30hz screen refresh, with a	*/
/* frameskip of 1 (draw only 1 frame every 2) to have a pseudo 60hz	*/
/* Screen refresh. 30 mean 35 real fps -> pseudo 70hz with frameskip 	*/
/*----------------------------------------------------------------------*/
#define Refresh_Delay	30	

SDL_Surface *screen;
SDL_Event event;
u8 *CharTable;
u8 *CharLookUp;
u8 *SpriteLookUp;
u8 *SpriteMemory;	/* Converted gfx */
u8 *SpriteRom;		/* 4 gfx roms */

u16 Palette [256];
u16 CharLookUpTable [2048];  
u16 SpriteLookUpTable[2048];

/*----------------------------------------------------------------------*/
/*	CPU will use this functions to access memory. They contain here	*/
/* code for eventual bank switching or I/O mapping. They act like a BUS	*/
/* These functions are provided and attached to CPU 'object' by user.	*/
/*----------------------------------------------------------------------*/
u8  RdMem8  (u16 Adr);			/* CPU 8bit Read Function 	*/
u16 RdMem16 (u16 Adr);			/* CPU 16bit Read Function	*/
void WrMem8 (u16 Addr,u8 Value);	/* CPU Write 8bit function	*/
void WrMem16(u16 Addr,u16 Value);	/* CPU Write 16bit function	*/
u8 * CPU1Memory;			/* CPU Own Memory,var   	*/
void CheckIrq (M6809CPU *CPU);		/* Use cpu to get cyclic interr.*/

/*----------------------------------------------------------------------*/
/* These variables are used as  IO port. Choosen depen. on emulated 	*/
/* video hardware: One joystick, no flip screen.4 live, easy game.	*/
/*----------------------------------------------------------------------*/
u8 IO_COIN=0xff;		/* 0w2400 Port: Coins,P1/P2 start	*/
u8 MachineIntMask= 0x00;	/* 0x2007 Port: Interrupt mask		*/
u8 PaletteBank =0x00;		/* 0x2200 Port: PaletteBank		*/
u8 IO_P1=0xff;			/* 0x2400 Port: Pad I/O 		*/
u8 DSW1 = 0xff;			/* 0x2500 Port: DipSwitch1 I/O 		*/
u8 DSW2 = 0xf1;			/* 0x2501 Port: DipSwitch2 I/O 		*/
u8 IO_FLIP_SCREEN = 0x2;	/* 0x2503 Port:Flip Screen I/O  	*/

int main() 
{
u8 Cycles,OpState;
M6809CPU CPU;

DisplayInit ();		/*	Init SDL Stuff */
AllocBuffer (&CharTable,8*4096,"CharTable");
AllocBuffer (&CharLookUp,0x100,"CharLookup");
AllocBuffer (&SpriteLookUp,0x100,"SpriteLookup");
AllocBuffer (&SpriteMemory,32768*4,"SpriteMemory");
AllocBuffer (&SpriteRom,0xffff,"SpriteRom");
AllocBuffer (&CPU1Memory,0xffff,"CPU1Memory");

/* US Rom Set 
*/
LoadRom (&CPU1Memory[0x6000],".\\mikie\\11c_n14.bin",0x2000);	// Code Rom 
LoadRom (&CPU1Memory[0x8000],".\\mikie\\12a_o13.bin",0x4000);	// Code Rom 
LoadRom (&CPU1Memory[0xC000],".\\mikie\\12d_o17.bin",0x4000);	// Code Rom 
LoadRom (&SpriteRom[0x0000],".\\mikie\\001",0x4000);		// Char Rom 
LoadRom (&SpriteRom[0x4000],".\\mikie\\003",0x4000);		// Sprite Rom 
LoadRom (&SpriteRom[0x8000],".\\mikie\\005",0x4000);		// Sprite Rom 
LoadRom (&SpriteRom[0xC000],".\\mikie\\007",0x4000);		// Sprite Rom 
/**/

/* Japanese rom set :-) 
LoadRom (&CPU1Memory[0x6000],".\\mikie\\11c_n14.bin",0x2000);	// Code Rom 
LoadRom (&CPU1Memory[0x8000],".\\mikie\\12a_o13.bin",0x4000);	// Code Rom 
LoadRom (&CPU1Memory[0xC000],".\\mikie\\12d_o17.bin",0x4000);	// Code Rom 
LoadRom (&SpriteRom[0x0000],".\\mikie\\f01_q01.bin",0x4000);	// Char Rom 
LoadRom (&SpriteRom[0x4000],".\\mikie\\f03_q03.bin",0x4000);	// Sprite Rom 
LoadRom (&SpriteRom[0x8000],".\\mikie\\h01_q05.bin",0x4000);	// Sprite Rom 
LoadRom (&SpriteRom[0xC000],".\\mikie\\h03_q07.bin",0x4000);	// Sprite Rom 
*/

/* Mikie High School graphiti rom set :-) 

LoadRom (&CPU1Memory[0x6000],".\\mikie\\11c_l14.bin",0x2000);	// Code Rom 
LoadRom (&CPU1Memory[0x8000],".\\mikie\\12a_m13.bin",0x4000);	// Code Rom 
LoadRom (&CPU1Memory[0xC000],".\\mikie\\12d_m17.bin",0x4000);	// Code Rom 
LoadRom (&SpriteRom[0x0000],".\\mikie\\f01_i01.bin",0x4000);	// Char Rom 
LoadRom (&SpriteRom[0x4000],".\\mikie\\f03_i03.bin",0x4000);	// Sprite Rom 
LoadRom (&SpriteRom[0x8000],".\\mikie\\h01_i05.bin",0x4000);	// Sprite Rom 
LoadRom (&SpriteRom[0xC000],".\\mikie\\h03_i07.bin",0x4000);	// Sprite Rom 
*/

Mikie_InitVideoHardWare ();			/* Init Video hardware */

CPU.ReadMem8 = &RdMem8;			/* Attach to CPU Object, read functions 		*/
CPU.ReadMem16 = &RdMem16;		/* CPU will use this func to perform a 16bit read  	*/
CPU.WriteMem8 = &WrMem8;		/* Attach to CPU Object, write functions 		*/
CPU.WriteMem16 = &WrMem16;		/* CPU will use this func to perform a 16bit write  	*/

Reset6809 (&CPU);
while (1)
	{
	OpState = Exec6809 (&CPU,&Cycles);
/*	if (OpState == 0x01) printf ("Unsupported/Illegal Opcode.\n");
	if (OpState == 0x10) printf ("Unsupported/Illegal 0x10 prefixed Opcode.\n");
	if (OpState == 0x11) printf ("Unsupported/Illegal 0x11 prefixed Opcode.\n");*/
	CheckIrq (&CPU);
	}
}

/*----------------------------------------------------------------------*/
/* Load Rom image at dst, from file filename of size size.		*/
/* Exits on error, with error message.					*/
/*----------------------------------------------------------------------*/
void LoadRom (u8 *dst,u8 *filename,s32 size)
{
FILE *ROM;
int lu;

ROM = fopen (filename,"rb");
if (ROM==NULL){printf ("Unable to open rom %s.\n",filename);exit (1);}
lu = fread (dst,1,size,ROM);
if (lu!=size){printf ("Unexpected end of %s ROM .\n",filename);exit (1);}
printf ("Loaded %s rom.\n",filename);
fclose (ROM);
}

/*----------------------------------------------------------------------*/
/* Video synchronisation. Just set Flag to 0. 1: Mean locked, waiting   */
/* for XX hz timer.							*/
/*----------------------------------------------------------------------*/
u32 TimerHandler (void)
{
RefreshFlag =0;			/* Timer signal receive, enable redraw	*/
return (Refresh_Delay);		/* Mean, restart timer 			*/
}

/*----------------------------------------------------------------------*/
/* Init Windows display, using SDL stuff.				*/
/* 1) Init Video + Timer sdl functions.					*/
/* 2) Get screen surface.						*/
/* 3) Set Windows title.						*/
/* 4) Set Timer at Refresh_Delay intervals.				*/
/*----------------------------------------------------------------------*/
void DisplayInit (void)
{
u32 my_timer;
if((SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER)==-1)){printf("Could not initialize SDL: %s.\n", SDL_GetError());exit(-1);}
screen = SDL_SetVideoMode(224, 256, 16, SDL_DOUBLEBUF);
SDL_WM_SetCaption("-=Mikie Emulator=-",NULL);
atexit(SDL_Quit);
my_timer = SDL_AddTimer(Refresh_Delay, TimerHandler,0);
if (my_timer == NULL) {printf("Could not initialize SDL Timer: %s.\n", SDL_GetError());exit(-1);}
}

/*----------------------------------------------------------------------*/
/* Setup mikie video emulation:						*/
/*	1) Create Color tables from Lookup proms			*/
/*	2) Generate PC palette from 3 color proms.			*/
/*	3) Load Tile data, convert them to PC usable format		*/
/*	4) Convert Sprite gfx roms to PC usable format.			*/
/*----------------------------------------------------------------------*/
void Mikie_InitVideoHardWare (void)
{
FILE *ROM;
u8 c;
s32 lu,i,j,x,y;
u8 *ptr, *src;
u8 *p;
u8 Tmp [64];

/* Load PROM Char lookup table */
LoadRom (CharLookUp,".\\mikie\\12h_d22.bin",0x100);

#define TOTAL_COLORS	2048
x=0;
for (i=0;i<TOTAL_COLORS/8;i++)
	{
	for (j=0;j<8;j++) 
		{
		CharLookUpTable [i+j*TOTAL_COLORS/8] = (CharLookUp [x]&0xf) + (32*j)+16 ; 
		}
	x++;
	}

/* Load PROM Sprite lookup table */
LoadRom (SpriteLookUp,".\\mikie\\f09_d18.bin",0x100);
x=0;
for (i=0;i<TOTAL_COLORS/8;i++)
	{
	for (j=0;j<8;j++) 
		{
		SpriteLookUpTable [(i+j*TOTAL_COLORS/8)] = (SpriteLookUp [x]&0xf) + (32*j); 
		}
	x++;
	}

/* Palette R */
ROM = fopen (".\\mikie\\01i_d19.bin","rb");
if (ROM==NULL){printf ("Unable to open rom 01i_d19.bin.\n");exit (1);}
for (i=0;i<0x100;i++)
	{
	lu = fread (&c,1,1,ROM);
	if (lu != 1) printf ("Unexpected end of rom 01i_d19.bin! \n");
	Palette [i] = c << 12;
	}
fclose (ROM);

/*Palette G */
ROM = fopen (".\\mikie\\03i_d21.bin","rb");
if (ROM==NULL){printf ("Unable to open rom 03i_d21.bin.\n");exit (1);}
for (i=0;i<0x100;i++)
	{
	lu = fread (&c,1,1,ROM);
	if (lu != 1) printf ("Unexpected end of rom 01i_d19.bin! \n");
	Palette [i] |= c << 7;
	}
fclose (ROM);

/*Palette B */
ROM = fopen (".\\mikie\\02i_d20.bin","rb");
if (ROM==NULL){printf ("Unable to open rom 02i_d20.bin.\n");exit (1);}
for (i=0;i<0x100;i++)
	{
	lu = fread (&c,1,1,ROM);
	if (lu != 1) printf ("Unexpected end of rom 01i_d19.bin! \n");
	Palette [i] |= c<<1;
	}
fclose (ROM);

/* Load Char Data, and convert them to usable 8bit format */
ROM = fopen (".\\mikie\\O11","rb");
if (ROM==NULL){printf ("Unable to open rom O11.\n");exit (1);}
ptr = (u8 *)CharTable;
for (i=0;i<16384;i++)
	{
	lu = fread (&c,1,1,ROM);
	if (lu != 1) printf ("Unexpected end of rom ! \n");
	*ptr = (u8) (c>>4);
	ptr++;
	*ptr = (u8) (c & 0xf);
	ptr++;
	}
fclose (ROM);
/* 90  Rotation */ 

p=(u8 *)CharTable;
for (i=0;i<512;i++)
	{
	for (y=0;y<8;y++)
		{
		for (x=0;x<8;x++)
			{
			Tmp[y*8+(7-x)] = p [y+x*8+(i*64)];
			}
		}
	for (y=0;y<8;y++)
		{
		for (x=0;x<8;x++)
			{
			p[y*8+x+(i*64)] = Tmp [y*8+x];
			}
		}
	}
/* Load Sprite Data, and convert them to usable 8bit format */
Mikie_ConvertGfxRoms ();
}

/*----------------------------------------------------------------------*/
/* Redraw mikie screen. The whole screen is redrawn, all sprites too.	*/
/* Dirty buffer can easily be implemented for serious speed up.		*/
/* Window can't provide 10 ms precise timer. So we use a 30 hz screen	*/
/* refresh. We only draw 1 frame on 2, to keep mikie 60 hz synch.	*/
/*----------------------------------------------------------------------*/
void UpdateVideo (void)
{
u32 size,i,o;
u16 tile;
u8 plt;

ToggleFlag ^=1;			/* Video redraw toggle variable 		*/
if (ToggleFlag) return;		/* Return 1 frame on 2				*/
while (RefreshFlag){}		/* We need redraw, but wait for 30hz sync. 	*/
SDL_LockSurface(screen);	/* Lock screen before accessing 		*/
RefreshFlag = 1;		/* Disable re-entrant redraw 			*/
TFrame++;			/* Inc Total frame only count drawn frame. 	*/
for (i=0;i<32;i++)
	{
	for (o=0;o<28;o++)
		{
		tile =  CPU1Memory[0x3c3f+(o*32)+(32-i)];
		plt =   CPU1Memory[0x383f+(o*32)+(32-i)];		
		tile += (plt & 0x20)<<3;
		DrawChar8x8Bloc (o,i,tile,plt); 
		}
	}    
DrawSprites ();
SDL_UnlockSurface(screen);
SDL_UpdateRect(screen, 0, 0,screen->w, screen->h);
}

/*----------------------------------------------------------------------*/
/*  8 BITS I/O OPERATIONS						*/
/*  ----------------------						*/
/*	CPU will use this functions to access memory. They contain here	*/
/* code for eventual bank switching or I/O mapping. They act like a BUS	*/
/* These functions are provided and attached to CPU 'object' by user.	*/
/*----------------------------------------------------------------------*/
u8  RdMem8  (u16 Addr)
{
if (Addr==0x2401) return (IO_P1);
if (Addr==0x2007) return (MachineIntMask); /* MachineIntMask = $0x2007 IO Address */
if (Addr==0x2403) return (IO_FLIP_SCREEN);				
if (Addr==0x2400) return (IO_COIN);
if (Addr==0x2500) return (DSW1);
if (Addr==0x2501) return (DSW2);
return (CPU1Memory [Addr]);
}

void WrMem8 (u16 Addr,u8 c)
{
if (Addr == 0x2007){MachineIntMask = c;return;} /* MachineIntMask = $0x2007 IO Address */
if (Addr == 0x2006) return; 			/* Flip screen port 0x2006 Port */
if (Addr == 0x2200){PaletteBank =c & 7;return;}	/* Palette selector = $0x2200 IO Address */
if (Addr>0x3FFF || Addr<0x2000) return; 	/* Rom write attempt */
if (Addr==0x2400) return; 			/* is Sound latch port No write here :-) */
CPU1Memory [Addr]=c;
}

/*----------------------------------------------------------------------*/
/*  16 BITS I/O OPERATIONS						*/
/*  ----------------------						*/
/*	CPU will use this functions to access memory. They contain here	*/
/* code for eventual bank switching or I/O mapping. They act like a BUS	*/
/* These functions are provided and attached to CPU 'object' by user.	*/
/*----------------------------------------------------------------------*/
/* Addr = MSB, Addr+1 = LSB, Big INDIANESS */

u16 RdMem16 (u16 Addr)
{
/* Like this to use the same mapper than 8bit I/O */
return ((RdMem8(Addr)<<8)+RdMem8(Addr+1)); 
}

void WrMem16 (u16 Addr,u16 c)
{
/* Like this to use the same mapper than 8bit I/O */
WrMem8(Addr,(u8)(c>>8));
WrMem8(Addr+1,(u8)(c&0xFF));
}

/*----------------------------------------------------------------------*/
/* PC Keyboard SDL handling. Simply invert bits, on KeyDown, KeyUp	*/
/* Event. Some mikie I/O are Active Low (0 mean pressed).		*/
/*----------------------------------------------------------------------*/
void SDL_React (void)
{
while( SDL_PollEvent( &event ) )
	{
	switch( event.type )
		{
		case SDL_KEYDOWN:
		switch( event.key.keysym.sym )
			{
                    	case SDLK_LEFT:
                        	IO_P1 &= (0xff^1);	/* Set bit1 to 0 (active) */
				IO_P1 |= 2;		/* Set bit2 to 1 ( inactive) */                        
                        break;
                    	case SDLK_RIGHT:
				IO_P1 &= (0xff^2);	/* Set bit2 to 0 (active) */
				IO_P1 |= 1;		/* Set bit1 to 1 ( inactive) */                        
                        break;
                    	case SDLK_UP:
				IO_P1 &= (0xff^4);	/* Set bit1 to 0 (active) */
				IO_P1 |= 8;		/* Set bit8 to 1 ( inactive) */                        
                        break;
                    	case SDLK_DOWN:
				IO_P1 &= (0xff^8);	/* Set bit1 to 0 (active) */
				IO_P1 |= 4;		/* Set bit4 to 1 ( inactive) */                        
                        break;
                    	case SDLK_LCTRL:		/* Bouton 1 */
				IO_P1 &= (0xff^0x10);	/* Set bit1 to 0 (active) */
                        break;
                    	case SDLK_LALT:			/* Bouton 2 */
				IO_P1 &= (0xff^0x20);	/* Set bit1 to 0 (active) */
                        break;
			case SDLK_c:	/* Simulate coin inserted ;-) */
				IO_COIN &= (0xff^1);
			break;
			case SDLK_1:	/* Simulate 1P Start ;-) */
				IO_COIN &= 0xf7;
			break;
			case SDLK_2:	/* Simulate 2P Start ;-) */
				IO_COIN &= 0xef;	/* Flip mode unsupported so be carefull with DSW2 ! */
			break;
                 	case SDLK_ESCAPE:
	                        printf ("Frames displayed : %d\n",TFrame);
	                        printf ("Time elapsed : %d\n",SDL_GetTicks()/1000);
	                        printf ("Average %d fps\n",TFrame/(SDL_GetTicks()/1000));
	                        SDL_Quit (); exit (0);
                        break;
                    	default:
                        break;
			}
		break;
		case SDL_KEYUP:
		switch( event.key.keysym.sym )
			{
                    	case SDLK_LEFT:
				IO_P1 |= 1;	/* Set bit1 to 1 ( inactive) */                        
                        break;
                    	case SDLK_RIGHT:
                        	IO_P1 |= 2;	/* Set bit1 to 1 ( inactive) */                        
                        break;
                    	case SDLK_UP:
                        	IO_P1 |= 4;	/* Set bit1 to 1 ( inactive) */                        
                        break;
                    	case SDLK_DOWN:
                        	IO_P1 |= 8;	/* Set bit1 to 1 ( inactive) */                        	
                        break;
			case SDLK_c:	/* Simulate coin inserted ;-) */
				IO_COIN |= 1;
			break;
			case SDLK_1:	/* Simulate P1 Start ;-) */
				IO_COIN |= 0x8;
			break;
			case SDLK_2:	/* Simulate P2 Start ;-) */
				IO_COIN |= 0x10;
			break;
                 	case SDLK_ESCAPE:
	                        SDL_Quit (); exit (0);
                        break;
                    	case SDLK_LCTRL:		/* Bouton 1 */
				IO_P1 |= 0x10;		/* Set bit1 to 0 (active) */
                        break;
                    	case SDLK_LALT:			/* Bouton 2 */
				IO_P1 |= 0x20;		/* Set bit1 to 0 (active) */
                        break;
                    	default:
                        break;
			}
		break;
       		default:
        	break;
		}
  	}
}

/*----------------------------------------------------------------------*/
/* Draw a 8*8 Tile. Tile can be flip x/y on the fly.Colors are 		*/
/* converted 'on the fly' from CharLookUpTable.				*/
/* Can easily be speed up, by using direct colors, and storing in PC	*/
/* ram already rotated tiles.						*/
/*----------------------------------------------------------------------*/
void DrawChar8x8Bloc (s32 x,s32 y,u32 nbr,u8 CM)
{
u8* src;
u16 *dst;
int i,o;
u16 color;
u8 CaraX [64];
u8 CaraY [64];
u8 t,tmp;

src = (u8 *) CharTable+(nbr*64);
dst = (u16 *) screen->pixels + ((y*8*224+(x*8)));

if (!(CM & 0x80))
	{
	for (i=0;i<8;i++)
		{
		for (o=0;o<8;o++)
			{
			CaraX [i*8+o] = src [i*8+(7-o)];
			}
		}
	src = CaraX;
	}

if (!(CM & 0x40))
	{
	for (i=0;i<8;i++)
		{
		for (o=0;o<8;o++)
			{
			CaraY [i*8+o] = src[((7-i)*8)+o];
			}
		}
	src = CaraY;
	}
tmp =(CM &0xf)+ (PaletteBank*16);
for (i=0;i<8;i++)
	{
	for (o=0;o<8;o++)
		{
		color = Palette [CharLookUpTable[(tmp<<4)+(*src)]];
		*dst = color;
		dst++;
		src++;
		}
	dst += (216);
	}
}
/*----------------------------------------------------------------------*/
/* Look at Sprite memory region, and see if needed to draw sprites.	*/
/*----------------------------------------------------------------------*/
void DrawSprites (void)
{
s32 i,o;
u8 sx,sy,p;
u16 offs = 0x2800;
u16 n;

for (i=0;i<36;i++)
	{
	sy = CPU1Memory [offs+3];		/* Get Y coordonnates */
	sy = 240-sy;
	sx = (244-16) - CPU1Memory [offs + 1]; 	/* -16 cause screen is 28 tile wide centered */
	n = (CPU1Memory [offs+2] & 0x3f)+((CPU1Memory [offs+2]&0x80)>>1)+((CPU1Memory [offs]&0x40)<<1);
	n+= ((CPU1Memory [offs+2] & 0x40)<<2);	/* Tile > N256	*/
	p = (CPU1Memory [offs] & 0x0f);		/* P is the Video color attribute */
	if (sx<(224-16) && sy<240)
		{
		if (!(CPU1Memory[offs] & 0x20)) DrawXinvSprite (sx,sy,n,p);
		else DrawSprite (sx,sy,n,p);
		}
	offs+=4;
	}
}

/*----------------------------------------------------------------------*/
/* Draw a 16*16 X inverted sprite. Colors are converted 'on the fly'	*/
/* from SpriteLookUpTable. Can easily be speed up, by using direct 	*/
/* colors.								*/
/*----------------------------------------------------------------------*/
void DrawXinvSprite (s32 x,s32 y,u16 n,u8 p)
{
int i,o;
u16* dst = (u16 *) screen->pixels + (y*224+x);
u8*  src = (u8 *) SpriteMemory + (256*n) + 15;
u16 color;
u16 tmp;

tmp =((p & 0xf)+(PaletteBank<<4))<<4 ; /*+PaletteBank*16; */
for (i=0;i<16;i++)
	{
	for (o=0;o<16;o++)
		{
		color = Palette [SpriteLookUpTable[tmp+(*src)]];
		if ((*src) !=0) *dst=color;
		dst++;
		src--;
		}
	src += 32;
	dst += (208);
	}
}
/*----------------------------------------------------------------------*/
/* Draw a 16*16 non inverted sprite. Colors are converted 'on the fly'	*/
/* from SpriteLookUpTable. Can easily be speed up, by using direct 	*/
/* colors.								*/
/*----------------------------------------------------------------------*/
void DrawSprite (s32 x,s32 y,u16 n,u8 p)
{
int i,o;
u16* dst = (u16 *) screen->pixels + (y*224+x);
u8*  src = (u8 *) SpriteMemory + 256*n;
u16 color;
u16 tmp;

tmp =(((p & 0xf) + PaletteBank*16))<<4 ; 
for (i=0;i<16;i++)
	{
	for (o=0;o<16;o++)
		{
		color = Palette [SpriteLookUpTable[tmp+(*src)]];
		if ((*src) !=0) *dst=color;
		dst++;
		src++;
		}
	dst += (208);
	}
}
/*----------------------------------------------------------------------*/
/* Handle Arcade Machine Interrupt.Cpu cycle counter is used as time	*/
/* value: each cycles takes a fixed duration. We use this like timer.	*/
/* IrqF => Cpu is 1.25Mhz. 1/60 sec = (1.25*10e6)/60 = 20588 cycles.	*/
/* Like this we can simulate a relative accurate 60 hz irq.		*/
/* In this routine, we check for I/O, and update PC screen.		*/
/*----------------------------------------------------------------------*/
void CheckIrq (M6809CPU *CPU)
{
if (CPU->Cycles > CPU->IrqF) 
	{
	if (MachineIntMask & 0x1) /* if interrupt enable */
		{
		Irq (CPU,0xfff8);
		} 	
	CPU->Cycles = 0;
	SDL_React ();
	UpdateVideo ();
	}
}
/*----------------------------------------------------------------------*/
/* These value, are ripped from mame mikie's driver. They are offset	*/
/* in BIT of each line (Cy) and Pixel (Cy) in a bit plan.		*/
/*----------------------------------------------------------------------*/
u16 Cx [16] ={ 32*8+0, 32*8+1, 32*8+2, 32*8+3, 16*8+0, 16*8+1, 16*8+2, 16*8+3,
		0, 1, 2, 3, 48*8+0, 48*8+1, 48*8+2, 48*8+3 };
			
u16 Cy [16] ={0*16, 1*16, 2*16, 3*16, 4*16, 5*16, 6*16, 7*16,
		32*16, 33*16, 34*16, 35*16, 36*16, 37*16, 38*16, 39*16 };
/*----------------------------------------------------------------------*/
/* Convert sprite rom, in a more usable format. (0000BBBB)		*/
/* Mikie roms are stored in a very odd format. We need decode them bit	*/
/* by bit.								*/
/*----------------------------------------------------------------------*/
void Mikie_ConvertGfxRoms (void)
{
int i,o,n;
u32 Plan0,Plan1,Plan2,Plan3,Ind,SpriteNbr;
u8 b0,b1,b2,b3;
u8 oct;

u8 STmp [16*16];

Plan3 = 0;
Plan2 = 4;
Plan1 = 262144;
Plan0 = 262148;

Ind=0;
for (n=0;n<256;n++)
	{
	SpriteNbr = n * 128*8;
	for (o=0;o<16;o++)
		{
		for (i=0;i<16;i++)
			{
			b0 = GetBit (Plan0 + Cy[o]+Cx[i] + SpriteNbr,SpriteRom);
			b1 = GetBit (Plan1 + Cy[o]+Cx[i] + SpriteNbr,SpriteRom);
			b2 = GetBit (Plan2 + Cy[o]+Cx[i] + SpriteNbr,SpriteRom);
			b3 = GetBit (Plan3 + Cy[o]+Cx[i] + SpriteNbr,SpriteRom);
			oct = (b3<<3) | (b2<<2) | (b1<<1) | (b0<<0); 	
			SpriteMemory [Ind++] = oct;
			}
		}
	Rotate16x16gfx (n);	// Rotate 90  
	}

Plan3 = 1*8+0;
Plan2 = 1*8+4;
Plan1 = 1*8+262144;
Plan0 = 1*8+262148;

for (n=0;n<256;n++)
	{
	SpriteNbr = n * 128*8;
	for (o=0;o<16;o++)
		{
		for (i=0;i<16;i++)
			{
			b0 = GetBit (Plan0 + Cy[o]+Cx[i] + SpriteNbr,SpriteRom);
			b1 = GetBit (Plan1 + Cy[o]+Cx[i] + SpriteNbr,SpriteRom);
			b2 = GetBit (Plan2 + Cy[o]+Cx[i] + SpriteNbr,SpriteRom);
			b3 = GetBit (Plan3 + Cy[o]+Cx[i] + SpriteNbr,SpriteRom);
			oct = (b3<<3) | (b2<<2) | (b1<<1) | (b0<<0); 	
			SpriteMemory [Ind++] = oct;
			}
		}
	Rotate16x16gfx (n+256);	// Rotate 90  
	}
}

/*----------------------------------------------------------------------*/
/* Return bit state in a given buffer.bit number is not byte number !	*/
/*----------------------------------------------------------------------*/
u8 GetBit (u32 bit_nbr,u8 *buffer)
{
u32 oct_nbr = bit_nbr/8;
u8  bit_n   = bit_nbr %8;

return ((buffer [oct_nbr]>>(7-bit_n)) & 1);
}

/*----------------------------------------------------------------------*/
/* 	Rotate 16*16 pic in inverse trigonometric sens.			*/
/*----------------------------------------------------------------------*/
void Rotate16x16gfx (u16 n)
{
u8 T [256];
u8*  src = (u8 *) SpriteMemory + 256*n ;
int x,y;

for (y=0;y<16;y++)
	{
	for (x=0;x<16;x++)
		{
		T [(15-x)+y*16] = src[x*16+y];
		}
	}
for (y=0;y<256;y++)
	{
	src [y] = T [y];
	}
}

/*----------------------------------------------------------------------*/
/* Allocate size bytes of memory, adjust var pointer.			*/
/* Exits on error with message.	If successfull, mem is cleared.		*/
/*----------------------------------------------------------------------*/
void AllocBuffer (u8 **var,u32 size,char *name)
{
*var = (u8*) malloc (size);
if (*var==NULL){printf ("Unable to allocate %s buffer.\n",name);exit (1);}
else printf ("Allocating %s buffer.\n",name);
memset (*var,0,size);		/* Clear memory */
}
